'''<h1>Library for surface x-ray diffraction simulations of superlattices</h1>
<p> The model is based on Fullertons algorithm for superlattices as
described in Phys. Rev. B vol. 45 p. 9292 (1992). 
'''
# Programmed by Matts Bjorck 20091215

import numpy as np
from . import sxrd
from .utils import f, rho
import time

from .sxrd import UnitCell, AtomGroup, Instrument, Slab, SymTrans

__pars__ = ['SLSample', 'SLStandard',  'UnitCell', 'Slab', 'AtomGroup', 
            'Instrument']

class SLSample:
    '''Class that models a multilayer sample on top of a 
    substrate according to Fullertons model as given in
    PRB ....
    '''
    def __init__(self, inst, bulk_slab, superlattice, unitcell, 
                 bulk_sym = []):
        self.set_bulk_slab(bulk_slab)
        self.set_bulk_sym(bulk_sym)
        self.superlattice = superlattice
        self.inst = inst
        self.set_unit_cell(unitcell)

    def set_bulk_slab(self, bulk_slab):
        '''Set the bulk unit cell to bulk_slab
        '''
        if type(bulk_slab) != type(sxrd.Slab()):
            raise TypeError("The bulk slab has to be a member of"
                            " class Slab")
        self.bulk = bulk_slab

    def set_unit_cell(self, unit_cell):
        '''Sets the unitcell of the sample
        '''
        if type(unit_cell) != type(sxrd.UnitCell(1.0, 1.0, 1.0)):
            raise TypeError("The bulk slab has to be a member"
                            " of class UnitCell")
        if unit_cell is None:
            unit_cell = sxrd.UnitCell(1.0, 1,.0, 1.0)
        self.unit_cell = unit_cell

    def set_bulk_sym(self, sym_list):
        '''Sets the list of allowed symmetry operations for the bulk

        sym_list has to be a list ([]) of symmetry elements from the
        class SymTrans
        '''
        # Type checking
        if type(sym_list) != type([]):
            raise TypeError("The surface symmetries has to contained"
                            " in a list")

        if sym_list == []:
            sym_list = [sxrd.SymTrans()]

        if min([type(sym) == type(sxrd.SymTrans()) for 
                sym in sym_list]) == 0:
            raise TypeError("All members in the symmetry list has to"
                            " be a memeber of class SymTrans")

        self.bulk_sym = sym_list

    def calc_i(self, h, k, l):
        '''Calculate the diffracted intensity from a superlattice.
        The diffracted intensity from the superlattice and the substrate
        are added. I.e. it is assumed that the films is not coherent with
        the substrate.
        '''
        bulk_i = np.abs(self.calc_fb(h, k, l))**2
        sl_i = np.abs(self.superlattice.calc_i(h, k, l))
        return (bulk_i + sl_i)*self.inst.inten

    def calc_fb(self, h, k, l):
        '''Calculate the structure factors from the bulk
        '''
        dinv = self.unit_cell.abs_hkl(h, k, l)
        x, y, z, el, u, oc, c = self.bulk._extract_values()
        oc = oc/float(len(self.bulk_sym))
        f = sxrd._get_f(self.inst, el, dinv)
        # Calculate the "shape factor" for the CTRs
        eff_thick = self.unit_cell.c/np.sin(self.inst.alpha*np.pi/180.0)
        alpha = (2.82e-5*self.inst.wavel*eff_thick/self.unit_cell.vol()*
                                              np.sum(f.imag,1))
        denom = np.exp(2.0*np.pi*1.0J*l)*np.exp(-alpha) - 1.0
        # Delta functions to remove finite size effect in hk plane
        delta_funcs=(abs(h - np.round(h)) < 1e-12)*(
            abs(k - np.round(k)) < 1e-12)
        # Sum up the uc struct factors
        f_u = np.sum(oc*f*np.exp(-2*np.pi**2*u*dinv[:, np.newaxis]**2)*
                     np.sum([np.exp(2.0*np.pi*1.0J*(
                            h[:,np.newaxis]*sym_op.trans_x(x, y) +
                            k[:,np.newaxis]*sym_op.trans_y(x, y) +
                            l[:,np.newaxis]*z [np.newaxis, :]))
                     for sym_op in self.bulk_sym], 0)
                    ,1)
        # Putting it all togheter
        fb = f_u/denom*delta_funcs
                       
        return fb
    
class Superlattice:
    '''Class that describe a superlattice, can be subclassed
    to implement different strain profiles, interdiffusion etc..
    '''
    def __init__(self, inst, unit_cell, a_slab, b_slab, 
                 a_sym = [], b_sym = []):
        self.a_slab = a_slab
        self.b_slab = b_slab
        if a_sym == []:
            self.a_sym = [sxrd.SymTrans()]
        else:
            self.a_sym = a_sym
        if b_sym == []:
            self.b_sym = [sxrd.SymTrans()]
        else:
            self.b_sym = b_sym
        self.unit_cell = unit_cell
        self.inst = inst

    def _extract_slab_values(self, slabs, sym):
        '''Extracts the necessary parameters for simulating
        a list of stacked slabs
        '''
        # Extract the parameters we need
        # the star in zip(*... transform the list elements to arguments
        xt, yt, zt, elt, ut, oct, ct = list(zip(*[slab._extract_values()
                                  for slab in slabs]))
       
        x = np. r_[xt]
        y = np.r_[yt]
        # scale and shift the slabs with respect to each other
        cn = np.cumsum(np.r_[0, ct])[:-1]
        z = np.concatenate([zs*c_s + c_cum
                            for zs, c_cum, c_s in zip(zt, cn, ct)])
        #el = reduce(lambda x,y:x+y, elt)
        el = np.r_[elt]
        u = np.r_[ut]
        oc = np.r_[oct]
        #print x,y,z, u
        t_lay =  sum(ct)
        return x, y, z, u, oc, el, t_lay

    def calc_fslab(self, slablist, sym, h, k, l):
        '''Calculate the structure factors from the bulk
        '''
        dinv = self.unit_cell.abs_hkl(h, k, l)
        x, y, z, u, oc, el, t_lay = self._extract_slab_values(slablist, 
                                                              sym)
        oc = oc/float(len(sym))
        f = sxrd._get_f(self.inst, el, dinv)
        # Sum up the uc struct factors
        f_u = np.sum(oc*f*np.exp(-2*np.pi**2*u*dinv[:, np.newaxis]**2)*
                     np.sum([np.exp(2.0*np.pi*1.0J*(
                            h[:,np.newaxis]*sym_op.trans_x(x, y) +
                            k[:,np.newaxis]*sym_op.trans_y(x, y) +
                            l[:,np.newaxis]*z [np.newaxis, :]))
                     for sym_op in sym], 0)
                    ,1)
        #return f_u, (z.max() - z.min())*np.ones(l.shape)
        return f_u, t_lay*np.ones(l.shape)

    def calc_fa(self, n, h, k, l):
        '''Calculate the strucutre factor for a a layer
        n is the thickness of the bilayer in units of slabs'''
        pass
    
    def calc_fb(self, n, h, k, l):
        '''Calcualte the structure factor for a b layer
        n is the thickness of the bilayer in units of slabs'''
        pass
    
    def calc_fsl(self, unit_cell, h, k, l):
        '''Calculate the strucutre factor for the entire 
        superlattice.
        '''
        raise NotImplementedError('calc_fsl has to be implemented in '
                                  'a Superlattices subclass')

class SLStandard(Superlattice):
    '''Class that implements a "standard" superlattice, no strain 
    included.
    '''
    _pars = {'sigmaa': 1e-12, 'sigmab':1e-12, 'repetitions':2, 'na':2, 
             'nb': 2,'a': 0.0, 'c':1e-12}

    def __init__(self, inst, unit_cell, a_slab, b_slab, 
                 a_sym = [], b_sym = []):
        Superlattice.__init__(self,  inst, unit_cell, a_slab, b_slab, 
                 a_sym = a_sym, b_sym = b_sym)
        [self._make_set_func(name, self._pars[name]) for name in 
         list(self._pars.keys())]
        [self._make_get_func(name) for name in list(self._pars.keys())]

    def calc_fa(self, n, h, k, l):
        f_slab, t_z = self.calc_fslab([self.a_slab]*n, self.a_sym, 
                                       h, k, l)
        return f_slab, t_z

    def calc_fb(self, n, h, k, l):
        f_slab, t_z = self.calc_fslab([self.b_slab]*n, self.b_sym,
                                        h, k, l)
        return f_slab, t_z

    def thick_prob(self, n_mean, stand_dev):
        # According to fullerton it's enough to include three
        # standard deviations in the averaging
        lower = np.floor(n_mean-3.0*stand_dev)
        lower = np.int(min(lower, n_mean - 1))
        # We can't have a negative thickness, altough we remove
        # the gaussian distribution How does this affect the theoretical
        # assumptions?
        if lower < 1:
            lower = 1
        upper = np.ceil(n_mean + 3.0*stand_dev)
        n = np.arange(lower, 
                      np.int(max(upper, n_mean + 1) + 1))
        #print 'n: ', n
        prob = np.exp(-(n - n_mean)**2/2.0/stand_dev**2)
        prob = prob/sum(prob)
        return n, prob

    def calc_i(self, h, k, l):
        ''' Function to calculate the form factor from a superlattice
        '''
        # Create the different thicknesses to avarage over
        na, pa = self.thick_prob(self.na, self.sigmaa)
        nb, pb = self.thick_prob(self.nb, self.sigmab)
        tmp = np.array([self.calc_fa(n, h, k, l) for n in na])
        fa = tmp[:, 0, :]
        ta = tmp[:, 1, :]
        tmp = np.array([self.calc_fb(n, h, k, l) for n in nb])
        fb = tmp[:, 0, :]
        tb = tmp[:, 1, :]
        #print pa.shape, fa.shape
        pa = pa[:, np.newaxis]
        pb = pb[:, np.newaxis]
        # Do the different averagning
        fafa = (pa*fa*fa.conj()).sum(0)
        fbfb = (pb*fb*fb.conj()).sum(0)
        fam = (pa*fa).sum(0)
        fbm = (pb*fb).sum(0)
        phia = (pa*np.exp(2*np.pi*1.0J*ta*l)*fa.conj()).sum(0)
        phib = (pb*np.exp(2*np.pi*1.0J*tb*l)*fb.conj()).sum(0)
        ta = (pa*np.exp(2*np.pi*1.0J*ta*l)).sum(0)
        tb = (pb*np.exp(2*np.pi*1.0J*tb*l)).sum(0)
        m = self.repetitions
        ksi = 2*np.pi*1.0J*l*self.a - (2*np.pi*l*self.c)**2/2.
        # Calculate the intensity
        int = (m*(fafa + 2.0*np.real(np.exp(ksi)*phia*fbm) + fbfb) + 
             2.0*np.real((np.exp(-ksi)*phib*fam/ta/tb + phia*fam/ta + 
                          phib*fbm/tb + np.exp(ksi)*phia*fbm)*(
            (m - (m + 1)*np.exp(2.0*ksi)*ta*tb + 
            (np.exp(2.0*ksi)*ta*tb)**(m + 1))/
            (1 - np.exp(2.0*ksi)*ta*tb)**2 - m)))
        return int

    def _make_set_func(self, name, val):
        '''Creates a function to set value for attribute with name'''
        def set_func(value):
            setattr(self, name, value)
        # Init the variable
        set_func(val)
        setattr(self, 'set' + name, set_func)
        

    def _make_get_func(self, name):
        ''' Creates a get function '''
        def get_func():
            return getattr(self, name)
        setattr(self, 'get' + name, get_func)
        

if __name__ == '__main__':
    from pylab import *

    inst = sxrd.Instrument(wavel = 0.77, alpha = 0.2)
    inst.set_inten(100.0)

    lay_a = sxrd.Slab()
    lay_a.add_atom('Sr', 'sr', 0.0, 0.0, 0.0, 0.001, 1.0)
    lay_a.add_atom('Ti', 'ti', 0.5, 0.5, 0.5, 0.001, 1.0)
    lay_a.add_atom('O1', 'o', 0.5, 0.0, 0.5, 0.001, 1.0)
    lay_a.add_atom('O2', 'o', 0.0, 0.5, 0.5, 0.001, 1.0)
    lay_a.add_atom('O3', 'o', 0.5, 0.5, 0.0, 0.001, 1.0)

    lay_b = sxrd.Slab(c = 1.0)
    lay_b.add_atom('La', 'la', 0.0, 0.0, 0.0, 0.001, 1.0, 1.0)
    lay_b.add_atom('Al', 'al', 0.5, 0.5, 0.5, 0.001, 1.0, 1.0)
    lay_b.add_atom('O1', 'o', 0.5, 0.5, 0.0, 0.001, 1.0, 1.)
    lay_b.add_atom('O2', 'o', 0.0, 0.5, 0.5, 0.001, 1.0, 1.)
    lay_b.add_atom('O3', 'o', 0.5, 0.0, 0.5, 0.001, 1.0, 1.)
    
    uc = sxrd.UnitCell(3.945, 3.945, 3.945, 90, 90, 90)

    sl = SLStandard(inst, uc, lay_b, lay_a)
    sl_sample = SLSample(inst, lay_a, sl, uc)
    sl.seta(0.0)
    sl.setc(0.00001)
    sl.setna(4.0)
    # Seems to have a lower limit of about 0.3 UC to work fine
    # with the calculation of thicknesses..
    sl.setsigmaa(0.3)
    sl.setnb(2.0)
    sl.setsigmab(0.3)
    sl.setrepetitions(10)
    l = np.arange(0.1, 3, 0.0011)
    h = 0.0*np.ones(l.shape)
    k = 0.0*np.ones(l.shape)
    int = sl_sample.calc_i(h, k, l)

    sample = sxrd.Sample(inst, lay_a, ([lay_b]*4 + [lay_a]*2)*10,
                    sxrd.UnitCell(3.945, 3.945, 3.945, 90, 90, 90))
    f_ref = sample.calc_f(h,k,l)
    int_ref = abs(f_ref)**2
    # Comparison between the normal sxrd model and the superlattice model.
    semilogy(l, int_ref)
    semilogy(l, int)
    legend(('sxrd model', 'sxrd_mult closed form'))
    xlabel('l [r.l.u.]')
    ylabel('Intensity')
    show()



from scipy import *
from genx.gui_logging import iprint

def IntElfield(theta,lamda,n,d):
    # Length of k-vector in vaccum
    k=2*math.pi/lamda
    # Calculates the wavevector in each layer
    Qj=2*k*sqrt(n[:,newaxis]**2-cos(theta*math.pi/180)**2)
    #print Qj
    # Fresnel reflectivity for the interfaces
    rp=(Qj[1:]-Qj[:-1])/(Qj[1:]+Qj[:-1])
    tp=1+rp
    #tp=2*Qj[1:]/(Qj[1:]+Qj[:-1])
    #print tp.shape
    #Beta=k*d*n[:-1]*sin(theta*math.pi/180)
    #print Beta.shape
    D=d[:,newaxis]*ones(theta.shape)
    #E_p=[1]
    #E_m=[0]
    E_p=[1*ones(theta.shape)]
    E_m=[0*ones(theta.shape)]
    #print E_p[0].shape
    #print D[0].shape
    #print Qj[0].shape
    #for i in range(0,len(rp)):
    #    E_p.append(1/tp[i]*(E_p[i]*exp(-1.0J*D[i]*Qj[i]/2)+E_m[i]*rp[i]*exp(1.0J*D[i]*Qj[i]/2)))
    #    E_m.append(1/tp[i]*(E_m[i]*exp(1.0J*D[i]*Qj[i]/2)+E_p[i]*rp[i]*exp(-1.0J*D[i]*Qj[i]/2)))
    for i in range(0,len(rp)):
        E_p.append(1/tp[i]*(E_p[i]*exp(-1.0J*D[i]*Qj[i]/2)+E_m[i]*rp[i]*exp(1.0J*D[i]*Qj[i]/2)))
        E_m.append(1/tp[i]*(E_m[i]*exp(1.0J*D[i]*Qj[i]/2)+E_p[i]*rp[i]*exp(-1.0J*D[i]*Qj[i]/2)))
    #print E_p
    E_p=array(E_p[1:])
    E_m=array(E_m[1:])
    return (E_p/E_p[-1],E_m/E_p[-1],Qj/2) 

def AmpElfield_q(k,kx,lamda,n,d):
    # Length of k-vector in vaccum
    k=2*math.pi/lamda
    # Calculates the wavevector in each layer
    #Qj=2*k*sqrt(n[:,newaxis]**2-cos(theta*math.pi/180)**2)
    Qj=2*sqrt(k**2*n[:,newaxis]**2-kx**2)
    # Fresnel reflectivity for the interfaces
    rp=(Qj[1:]-Qj[:-1])/(Qj[1:]+Qj[:-1])
    #print rp
    tp=1+rp
    #tp=2*Qj[1:]/(Qj[1:]+Qj[:-1])
    #print tp.shape
    #Beta=k*d*n[:-1]*sin(theta*math.pi/180)
    #print Beta.shape
    D=d[:,newaxis]*ones(kx.shape)
    #print D.shape
    #print rp.shape
    #E_p=[1]
    #E_m=[0]
    # old version
    E_p=[1*ones(kx.shape)]
    E_m=[0*ones(kx.shape)]
    for i in range(0,len(rp)):
        E_p.append(1/tp[i]*(E_p[i]*exp(-1.0J*D[i]*Qj[i]/2)+E_m[i]*rp[i]*exp(1.0J*D[i]*Qj[i]/2)))
        E_m.append(1/tp[i]*(E_m[i]*exp(1.0J*D[i]*Qj[i]/2)+E_p[i]*rp[i]*exp(-1.0J*D[i]*Qj[i]/2)))
    #Added 020905 for testing another T amplitude (The Transmitted amp. trough the interface)
    #print E_p
    # End old version
    E_p=array(E_p)
    E_m=array(E_m)
    #print E_p.shape
    #print Qj.shape
    dtemp=r_[d,0]
    #print dtemp
    dtemp=dtemp[:,newaxis]*ones(kx.shape)
    E_p=E_p*exp(-1.0J*dtemp*Qj/2)
    E_m=E_m*exp(1.0J*dtemp*Qj/2)
    E_p=list(E_p)
    E_m=list(E_m)
    # End addition
    E_p.reverse()
    E_m.reverse()
    Qj=list(Qj)
    Qj.reverse()
    Qj=array(Qj)
    # Removed 020905 as said above and replaced
    E_p=array(E_p) #Old version
    #E_p=array(E_p)
    # End addition
    E_m=array(E_m)
    q=Qj/2 # Old ver. replaced 020905 with:
    #return (E_p[:-1]/E_p[0],E_m[:-1]/E_p[0],q[:-1]) # Last correspond to kz
    #testar amplitud i lager j+1 istallet
    return (E_p[1:]/E_p[0],E_m[1:]/E_p[0],q[1:]) # Last correspond to kz

def AmpElfield2(kx,k,n,z):
    kz=sqrt(n[:,newaxis]**2*k**2-kx**2)
    r=(kz[:-1]-kz[1:])/(kz[:-1]+kz[1:])
    t=1+r
    X=0*ones(kx.shape)
    for i in range(len(n)-2,-1,-1):
        X=exp(-2j*kz[i]*z[i])*(r[i]+X*exp(2j*kz[i+1]*z[i]))/(1+r[i]*X*exp(2j*kz[i+1]*z[i]))
        iprint(i)
    # X=reflected amplitude...
    r=(kz[1:]-kz[:-1])/(kz[:-1]+kz[1:])
    t=1+r
    R=[X]
    T=[1*ones(X.shape)]
    for i in range(0,len(n)-1,1):
        R.append(1/t[i]*(T[i]*r[i]*exp(-1.0j*(kz[i+1]+kz[i])*z[i])+R[i]*exp(-1.0j*(kz[i+1]-kz[i])*z[i])))
        T.append(1/t[i]*(T[i]*exp(1.0j*(kz[i+1]-kz[i])*z[i])+R[i]*r[i]*exp(1.0j*(kz[i+1]+kz[i])*z[i])))
    #R[len(n)-1]=0*ones(X.shape)
    R=array(R)
    T=array(T)
    return (T,R,kz)

from .xrmr import dot2, inv2

def AmpElfield_test(th, lam, n, d, dz = 0.1):
    ''' Added 20110317 to see if I can do calcs of the fields inside a multilayer '''
    kz = 2*pi/lam*sin(th*pi/180)
    k = 2*pi/lam
    #n = r_[n, n[-1]]
    #d = r_[d, d[-1]]
    kz=sqrt((n[:,newaxis]**2 - 1)*k**2+kz**2)
    #print kz.shape, d.shape
    a = exp(-1.0J*kz*d[:, newaxis])[1:,:]
    #print a.shape
    #print kz[newaxis, newaxis, :, :].shape
    r = (kz[:-1] -kz[1:])/(kz[:-1] + kz[1:])
    M = 1/kz[newaxis, newaxis, :-1, :]/2*array([[(kz[1:] + kz[:-1])*a, (kz[:-1] - kz[1:])/a],
                      [(kz[:-1] - kz[1:])*a, (kz[1:] + kz[:-1])/a]])
    #M = array([[a, r*a],[r/a, 1/a]])
    M = rollaxis(M, 2, 0)[::,:,:,:]
    # "quick" version
    #MM = reduce(dot2, M) 
    MMcum = [M[0]]
    for i in range(1, M.shape[0]):
        MMcum.append(dot2(MMcum[-1], M[i]))
    MM = MMcum[-1]
    
    R0 = MM[1,0]/MM[0,0]
    TN = 1/M[0,0]
    R = abs(R0)**2
    T = abs(TN)**2
    
    # Calc el. fileds as a function of z
    Rlay = R0
    Tlay = 1.0
    z = array([0])
    iprint('R0 ',R0.shape)
    E = array(abs(R0 + 1.0))[:, newaxis]*ones((th.shape[0],1))
    #E = array([])
    iprint(kz.shape)
    iprint((th.shape[0], 1))
    iprint(ones((th.shape[0], 1)).shape)
    iprint(E.shape)
    for di, kzi, M, in zip(d[1:], kz[1:], MMcum):
         Mlay = inv2(M)
         Rlay_new = Mlay[0,0]*Tlay + Mlay[0,1]*Rlay
         Tlay_new = Mlay[1,0]*Tlay + Mlay[1,1]*Rlay
         znew = arange(0, di, dz)
         z = append(z, znew + z[-1])
         #print kzi.shape
         a = exp(-1.0J*kzi[:, newaxis]*(di - znew))
         #print (abs(Rlay_new[:, newaxis]*a + Tlay_new[:, newaxis]/a)**2).shape
         E = c_[E, abs(Rlay_new[:, newaxis]*a + Tlay_new[:, newaxis]/a)]
         Rlay = Rlay_new
         Tlay = Tlay_new
    
         
    #print MM.shape
    iprint(E.shape)
    #E = E.reshape((th.shape[0], z.shape[0]))
    return (T, R, z, E)

if __name__=='__main__':

    #R=[]
    #for x in arange(0,2,0.01):
    #    (E_p,E_m)=IntElfield(x,1.54,array([1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1]),array([0,80,20,80,20,80,20]))
    #    R.append(abs(E_m[-1]/E_p[-1])**2)
    import pylab
    theta=arange(0,1+0.001666670,0.005)
    #(E_p,E_m,Qj)=IntElfield(theta,1.54,array([1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1]),array([0,80,20,80,20,80,20]))
    #(E_p,E_m,Qj)=IntElfield(theta,1.540562,array([1-.15E-04/2+0.35E-06j/2,1-.45E-04/2+0.60E-05j/2,1]),array([0,1000]))
    from .paratt import Refl
    #c=Refl(theta,1.54,array([1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j,1]),array([80,20,80,20,80,20]),0)
    c=Refl(theta,1.54,array([1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j,1]),array([0,1000,0]),array([0,0,0]))
    #gplt.plot(theta,log10(abs(E_m[-1]/E_p[-1])**2),theta,log10(abs(c**2)))
    #f=f=open('Specrefl.dat','r')
    #t=io.read_array(f)
    #t=transpose(t)
    #gplt.plot(theta,log10(abs(E_m[-1]/E_p[-1])**2),t[0],log10(t[1]))
    k=2*math.pi/1.54
    n=[1,1-2.24e-5+2.89e-6j,1-7.57e-6+1.73e-7j]
    n=array(n)
    z=array([0,-1000])
    #(T,R,k)=AmpElfield2(k*cos(theta*math.pi/180),k*ones(theta.shape), n,z)
    #(T,R,k)=IntElfield(theta,1.54, n, array([0,1000]))
    (T, R, zc, E)=AmpElfield_test(theta,1.54, n, array([0,1000,0]))
    #print T[0]
    pylab.subplot(211)
    pylab.plot(theta, log10(abs(R)), theta, log10(abs(c)), '.-')
    pylab.ylabel('R')
    pylab.xlabel('theta [deg]')
    pylab.legend(('Abeles', 'Parratt'))
    pylab.subplot(212)
    pylab.contourf(theta, zc, E.transpose())
    #print zc.shape, zc[-1]
    #print log10(E[0,:]).shape
    #pylab.plot(zc, (E[1,:]))
    pylab.show()



# Functions for instrumental resolution corrections and area of illumination
# these are to be included in the Models class and Refl class
# Note that these function assumes a Gaussian spread in both the angular as well
# as in real space

# THis one really needs scipy
from scipy import *
from numpy import *
from scipy.special import erf

# Area correction
def GaussArea(alpha,s1,s2,sigma_x):
    alpha=alpha*(abs(alpha)>1e-7)+1e-7*(abs(alpha)<=1e-7)
    sinalpha=sin(alpha*pi/180)
    A=sqrt(pi/2.0)*sigma_x*(erf(s2*sinalpha/sqrt(2.0)/sigma_x)+erf(s1*sinalpha/sqrt(2.0)/sigma_x))/sinalpha
    return A

# Intensity correction Gaussian beamprofile
def GaussIntensity(alpha,s1,s2,sigma_x):
    sinalpha=sin(alpha*pi/180)
    return (erf(s2*sinalpha/sqrt(2.0)/sigma_x)+erf(s1*sinalpha/sqrt(2.0)/sigma_x))/2.0

# Diffuse correction: Area corr
def GaussDiffCorrection(alpha,s1,s2,sigma_x):
    return GaussArea(alpha,s1,s2,sigma_x)*GaussIntensity(alpha,s1,s2,sigma_x)

# Specular foorprintcorrections square beamprofile
def SquareIntensity(alpha,slen,beamwidth):
    F=slen/beamwidth*sin(alpha*pi/180)
    return where(F<=1.0,F,ones(F.shape))

# Function to calculate the instrumental resolution in incomming, alpha, and
# outgoing, beta, directions
def ResVariance(alpha,beta,sigma_x,sigma_xp,samplewidth,slitwidth,DetGCdist):
    alpha=alpha*pi/180.0
    beta=beta*pi/180.0
    sigma_sample2=samplewidth**2/12
    sigma_slit2=slitwidth**2/12
    sigma_t2=sigma_x**2*sigma_sample2*sin(beta)**2/(sigma_sample2*sin(alpha)**2+sigma_x**2)
    sigma_beta=sqrt(sigma_slit2+sigma_t2)/DetGCdist
    sigma_alpha=ones(beta.shape)*sigma_xp
    return (sigma_alpha,sigma_beta)

# Function to convolve the tth scan with a gaussian for simulating
# the instrumental resolution
def SpecularRes(tth,I,sigma_alpha,sigma_beta,points):
    # cutoff is between 0 and 1 and describes how much of the gaussian should
    # be incorporated in the resolution
    sigmainv2=(1/sigma_alpha**2+1/sigma_beta**2)/4
    #print sigmainv2
    #tthcutoff=max(sqrt(-2.0/sigmainv2*log(cutoff)))
    dtth=arange(-points,points+1)*(tth[1]-tth[0])
    Iconv=ones(I.shape)
    resfunc=exp(-(dtth*pi/180.0)**2/2*min(sigmainv2))
    resfunc=resfunc/sum(resfunc)
    #print sum(resfunc)
    Iconv=convolve(I,resfunc,mode=1)

    return Iconv

####################################################################
## Resolution Functions (Normal Distributed=Gaussian)
#####################################################################

#Full Convlutions - vayring resolution

# Function to create a 1D vector for the resolution with the 
# positions to calculate the reflectivity Qret and the weight
# of each point weight
# Inputs: Q - the Q values 
#         dQ - the resolution
#         points - the number of points for the convolution
#         range how far the gaussian should be convoluted
def ResolutionVector(Q,dQ,points,range=3):
    #if type(dQ)!=type(array([])):
    #    dQ=dQ*ones(Q.shape)
    Qstep=2*range*dQ/points
    Qres=Q+(arange(points)-(points-1)/2)[:,newaxis]*Qstep
    
    weight=1/sqrt(2*pi)/dQ*exp(-(transpose(Q[:,newaxis])-Qres)**2/(dQ)**2/2)
    Qret = Qres.flatten()#reshape(Qres,(1,Qres.shape[0]*Qres.shape[1]))[0]
    #print Qres
    #print Qres.shape
    #print Qret.shape
    return (Qret,weight)

# Include the resolution with Qret and weight calculated from ResolutionVector
# and I the calculated intensity at each point. returns the intensity
def ConvoluteResolutionVector(Qret,I,weight):
    Qret2 = Qret.reshape(weight.shape[0], weight.shape[1])
    #print Qret.shape,weight.shape
    I2 = I.reshape(weight.shape[0], weight.shape[1])
    #print (I*weight).shape,Qret.shape
    norm_fact = trapz(weight, x = Qret2, axis = 0)
    Int = trapz(I2*weight, x = Qret2, axis = 0)/norm_fact
    #print Int.shape
    return Int

# Fast convlution - constant resolution
# constant spacing between data!
def ConvoluteFast(Q,I,dQ,range=3):
    Qstep=Q[1]-Q[0]
    resvector=arange(-range*dQ,range*dQ+Qstep,Qstep)
    weight=1/sqrt(2*pi)/dQ*exp(-(resvector)**2/(dQ)**2/2)
    Iconv=convolve(r_[ones(resvector.shape)*I[0],I,ones(resvector.shape)*I[-1]], weight/weight.sum(), mode=1)[resvector.shape[0]:-resvector.shape[0]]
    return Iconv

# Fast convolution - varying resolution
# constant spacing between the dat.
def ConvoluteFastVar(Q,I,dQ,range=3):
    Qstep = Q[1]-Q[0]
    steps = max(dQ*ones(Q.shape))*range/Qstep
    weight = 1/sqrt(2*pi)/dQ*exp(-(Q[:,newaxis]-Q)**2/(dQ)**2/2)
    Itemp = I[:,newaxis]*ones(I.shape)
    norm_fact = trapz(weight, axis = 0)
    Int = trapz(Itemp*weight,axis = 0)/norm_fact
    return Int


# Add functions for determining s1,s2,sigma_x,sigma_xp




'''
Just a library of the different eventhandler needed in the GUI
File started by: Matts Bjorck

$Rev::                                  $:  Revision of last commit
$Author::                               $:  Author of last commit
$Date::                                 $:  Date of last commit
'''
from . import version
__version__ = version.__version__

import _thread, time
import wx, os, traceback
from io import StringIO
import wx.adv
from wx.lib.wordwrap import wordwrap
import webbrowser
import numpy as np

from . import model as modellib
from . import solvergui, help
from . import filehandling as io
from .gui_logging import iprint

manual_url = 'http://genx.sourceforge.net/doc/'
homepage_url = 'http://genx.sf.net'

def get_pages(frame):
    # Get all plot panel objects in GUI
    pages = []
    for page in frame.plot_notebook.GetChildren():
        pages+=page.GetChildren()
    if frame.sep_plot_notebook is not frame.plot_notebook:
        for page in frame.sep_plot_notebook.GetChildren():
            pages+=page.GetChildren()

    # pages = [frame.plot_data, frame.plot_fom, frame.plot_pars,\
    #             frame.plot_fomscan]
    return pages

def set_title(frame):
    filepath, filename = os.path.split(frame.model.filename)
    if filename != '':
        if frame.model.saved:
            frame.SetTitle(filename + ' - ' + filepath + ' - GenX '\
                + __version__)
        else:
            frame.SetTitle(filename + '* - ' + filepath + ' - GenX '\
                + __version__)
    else:
        frame.SetTitle('GenX ' + __version__)
        
def models_changed(frame, event):
    '''models_changed(frame, event) --> None
    
    callback when something has changed in the model so that the 
    user can be made aware that the model needs saving.
    '''
    try:
        frame.model.saved = not event.permanent_change
    except AttributeError:
        frame.model.saved = False
    else:
        frame.plugin_control.OnGridChanged(event)
    set_title(frame)

def new(frame, event):
    '''
    new(frame, event) --> None
    
    Event handler for creating a new model
    '''
    #print "Event handler: new"
    if not frame.model.saved:
        ans = ShowQuestionDialog(frame, 'If you continue any changes in' 
                                 ' your model will not be saved.', 
                                 'Model not saved')
        if not ans:
           return
    
    # Reset the model - remove everything from the previous model
    frame.model.new_model()
    # Update all components so all the traces are gone.
    _post_new_model_event(frame, frame.model, desc='Fresh model')
    frame.plugin_control.OnNewModel(None)
    frame.main_frame_statusbar.SetStatusText('New model created', 1)
    set_title(frame)
    frame.model.saved = True
    
def open(frame, event):
    '''
    open(frame, event) --> None
    
    Event handler for opening a model file...
    '''
    # Check so the model is saved before quitting
    if not frame.model.saved:
        ans = ShowQuestionDialog(frame, 'If you continue any changes in' 
                                 ' your model will not be saved.', 
                                 'Model not saved')
        if not ans:
           return
     
    dlg = wx.FileDialog(frame, message="Open", defaultFile="",\
                        wildcard="GenX File (*.hgx;*.gx)|*.hgx;*.gx",\
                         style=wx.FD_OPEN #| wx.FD_CHANGE_DIR
                       )
    if dlg.ShowModal() == wx.ID_OK:
        path = dlg.GetPath()
        open_model(frame, path)
        
    dlg.Destroy()
    

def open_model(frame, path):
    #print "open_model"
    frame.model.new_model()
    frame.paramter_grid.PrepareNewModel()
    # Update all components so all the traces are gone.
    #_post_new_model_event(frame, frame.model)
    try:
        io.load_file(path, frame.model, frame.solver_control.optimizer, frame.config)
    except modellib.IOError as e:
        ShowModelErrorDialog(frame, e.__str__())
    except Exception as e:
        outp = StringIO()
        traceback.print_exc(200, outp)
        val = outp.getvalue()
        outp.close()
        iprint('Error in loading the file ', path, '. Pyton traceback:\n ',val)
        ShowErrorDialog(frame, 'Could not open the file. Python Error:\n%s' % (val,))
        return
    try:
        [p.ReadConfig() for p in get_pages(frame)]
    except Exception as e:
        outp = StringIO()
        traceback.print_exc(200, outp)
        val = outp.getvalue()
        outp.close()
        iprint('Error in loading config for the plots. Pyton traceback:\n ',val)
        ShowErrorDialog(frame, 'Could not read the config for the plots. Python Error:\n%s' % (val,))
    try:
        frame.paramter_grid.ReadConfig()
    except Exception as e:
        outp = StringIO()
        traceback.print_exc(200, outp)
        val = outp.getvalue()
        outp.close()
        iprint('Error in loading config for parameter grid. Pyton traceback:\n ', val)
        ShowErrorDialog(frame, 'Could not read the config for the parameter grid. Python Error:\n%s' % (val,))
    else:
        # Update the Menu choice
        frame.main_frame_menubar.mb_view_grid_slider.Check(frame.paramter_grid.GetValueEditorSlider())
    # Letting the plugin do their stuff...
    try:
        frame.plugin_control.OnOpenModel(None)
    except Exception as e:
        outp = StringIO()
        traceback.print_exc(200, outp)
        val = outp.getvalue()
        outp.close()
        ShowErrorDialog(frame, 'Problems when plugins processed model.'\
                    ' Python Error:\n%s'%(val,))
    frame.main_frame_statusbar.SetStatusText('Model loaded from file',\
                                                1)
    # Post an event to update everything else
    _post_new_model_event(frame, frame.model)
    # Needs to put it to saved since all the widgets will have 
    # been updated
    frame.model.saved = True
    set_title(frame)
    
def on_new_model(frame, event):
    '''
    on_new_model(frame, event) --> None
    
    Callback for NEW_MODEL event. Used to update the script for
    a new model i.e. put the string to the correct value. 
    '''
    #print "Callback: on_new_model"
    # Set the string in the script_editor
    frame.script_editor.SetText(event.GetModel().get_script())
    # Let the solvergui do its loading and updating:
    frame.solver_control.ModelLoaded()
    # Lets update the mb_use_toggle_show Menu item
    frame.main_frame_menubar.mb_use_toggle_show.Check(
        frame.config.get_boolean('data handling', 'toggle show'))
    try:
        val = frame.config.get_boolean('parameter grid', 'auto sim')
    except io.OptionError:
        iprint('Could not locate option parameters.auto sim')
        frame.main_frame_menubar.mb_fit_autosim.Check(True)
    else:
       frame.main_frame_menubar.mb_fit_autosim.Check(val)

    # Let other event handlers recieve the event as well
    event.Skip()

def update_for_save(frame):
    """Updates the various objects for a save"""
    frame.model.set_script(frame.script_editor.GetText())
    # Save the current state of autosim to the config
    frame.config.set('parameter grid', 'auto sim', frame.main_frame_menubar.mb_fit_autosim.IsChecked())

def save(frame, event):
    '''
    save(frame, event) --> None
    
    Event handler for saving a model file ...
    '''
    update_for_save(frame)
    fname = frame.model.get_filename()
    # If model hasn't been saved
    if fname == '':
        # Proceed with calling save as
        save_as(frame, event)
    else:
        # If it has been saved just save it
        try:
            io.save_file(fname, frame.model, frame.solver_control.optimizer, frame.config)
        except modellib.IOError as e:
            ShowModelErrorDialog(frame, e.__str__())
        except Exception as e:
            outp = StringIO()
            traceback.print_exc(200, outp)
            val = outp.getvalue()
            outp.close()
            ShowErrorDialog(frame, 'Could not save the file. Python Error: \n%s' % (val,))
        set_title(frame)
        
    frame.main_frame_statusbar.SetStatusText('Model saved to file', 1)
    

def save_as(frame, event):
    '''save_as(frame, event) --> None
    
    Event handler for save as ...
    '''
    dlg = wx.FileDialog(frame, message="Save As", defaultFile="",
                        wildcard="HDF5 GenX File (*.hgx)|*.hgx|GenX File (*.gx)|*.gx",
                        style=wx.FD_SAVE #| wx.FD_CHANGE_DIR
                        )
    if dlg.ShowModal() == wx.ID_OK:
        update_for_save(frame)
        fname = dlg.GetPath()
        base, ext = os.path.splitext(fname)
        if ext == '':
            ext = '.hgx'
        fname = base + ext
        result = True
        if os.path.exists(fname):
            filepath, filename = os.path.split(fname)
            result = ShowQuestionDialog(frame, 'The file %s already exists. Do you wish to overwrite it?' % filename,
                                        'Overwrite?')
        if result:
            try:
                io.save_file(fname, frame.model, frame.solver_control.optimizer,\
                        frame.config)
            except modellib.IOError as e:
                ShowModelErrorDialog(frame, e.__str__())
            except Exception as e:
                outp = StringIO()
                traceback.print_exc(200, outp)
                val = outp.getvalue()
                outp.close()
                ShowErrorDialog(frame, 'Could not save the file. Python Error:\n%s'%(val,))
            set_title(frame)
        
    dlg.Destroy()
    
def export_data(frame, event):
    '''export_data(frame, event) --> None
    
    exports the data to one file per data set with a basename with
    extention given by a save dialog.
    '''
    dlg = wx.FileDialog(frame, message="Export data", defaultFile="",\
                        wildcard="Dat File (*.dat)|*.dat",\
                         style=wx.FD_SAVE | wx.FD_CHANGE_DIR
                       )
    if dlg.ShowModal() == wx.ID_OK:
        try:
            frame.model.export_data(dlg.GetPath())
        except modellib.IOError as e:
            ShowModelErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText(\
                    'Error when exporting data', 1)
        except Exception as e:
            ShowErrorDialog(frame, str(e), 'export data - model.export_data')
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
        else:
            frame.main_frame_statusbar.SetStatusText('Data exported', 1)
        
    dlg.Destroy()
    
def export_script(frame, event):
    '''export_script(frame, event) --> None
    
    Exports the script to a python file given by a filedialog.
    '''
    dlg = wx.FileDialog(frame, message="Export data", defaultFile="",\
                        wildcard="Python File (*.py)|*.py",\
                         style=wx.FD_SAVE | wx.FD_CHANGE_DIR
                       )
    if dlg.ShowModal() == wx.ID_OK:
        fname = dlg.GetPath()
        base, ext = os.path.splitext(fname)
        if ext == '':
            ext = '.py'
        fname = base + ext
        result = True
        if os.path.exists(fname):
            filepath, filename = os.path.split(fname)
            result = ShowQuestionDialog(frame, \
            'The file %s already exists. Do you wish to overwrite it?'%filename\
            , 'Overwrite?')
        if result:
            try:
                #frame.model.export_script(dlg.GetPath())
                frame.model.export_script(fname)
            except modellib.IOError as e:
                ShowModelErrorDialog(frame, str(e))
                frame.main_frame_statusbar.SetStatusText(\
                        'Error when exporting script', 1)
                return
            except Exception as e:
                ShowErrorDialog(frame, str(e),\
                                    'export script - model.export_script')
                frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
                return
            else:
                frame.main_frame_statusbar.SetStatusText(\
                                                'Script exported to file', 1)
            
    dlg.Destroy()
    
def export_table(frame, event):
    '''export_table(frame, event) --> None
    
    Exports the table to a dat file given by a filedialog.
    '''
    dlg = wx.FileDialog(frame, message="Export data", defaultFile="",\
                        wildcard="Table File (*.tab)|*.tab",\
                         style=wx.FD_SAVE | wx.FD_CHANGE_DIR
                       )
    if dlg.ShowModal() == wx.ID_OK:
        fname = dlg.GetPath()
        base, ext = os.path.splitext(fname)
        if ext == '':
            ext = '.tab'
        fname = base + ext
        result = True
        if os.path.exists(fname):
            filepath, filename = os.path.split(fname)
            result = ShowQuestionDialog(frame, \
            'The file %s already exists. Do you wish to overwrite it?'%filename\
            , 'Overwrite?')
        if result:
            try:
                #frame.model.export_table(dlg.GetPath())
                frame.model.export_table(fname)
            except modellib.IOError as e:
                ShowModelErrorDialog(frame, str(e))
                frame.main_frame_statusbar.SetStatusText(\
                        'Error when exporting table', 1)
                return
            except Exception as e:
                ShowErrorDialog(frame, str(e),\
                                    'export table - model.export_table')
                frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
                return
            else:
                frame.main_frame_statusbar.SetStatusText(\
                                                'Table exported to file', 1)
            
    dlg.Destroy()
    
def import_script(frame, event):
    '''import_script(frame, event) --> None
    
    imports a script from the file given by a file dialog box
    '''
    dlg = wx.FileDialog(frame, message="Import script", defaultFile="",\
                    wildcard="Python files (*.py)|*.py|All files (*.*)|*.*",\
                         style=wx.FD_OPEN | wx.FD_CHANGE_DIR
                       )
    if dlg.ShowModal() == wx.ID_OK:
        try:
            frame.model.import_script(dlg.GetPath())
            #frame.model.import_script(fname)
        except modellib.IOError as e:
            ShowModelErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText(\
                    'Error when importing script', 1)
            return
        except Exception as e:
            ShowErrorDialog(frame, str(e),\
                                'import script - model.import_script')
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
            return
        try:
            frame.plugin_control.OnOpenModel(None)
        except Exception as e:
            outp = StringIO()
            traceback.print_exc(200, outp)
            val = outp.getvalue()
            outp.close()
            ShowErrorDialog(frame, 'Problems when plugins processed model.'\
                    ' Python Error:\n%s'%(val,))
        else:
            frame.main_frame_statusbar.SetStatusText(\
                                                'Script imported from file', 1)
    dlg.Destroy()
    # Post event to tell that the model has changed
    _post_new_model_event(frame, frame.model)
    
def import_data(frame, event):
    ''' import_data(frame, event) -->None
    
    callback to import data into the program
    '''
    # Reuse of the callback in the datalist.DataController
    try:
        frame.data_list.eh_tb_open(event)
    except Exception as e:
            ShowErrorDialog(frame, str(e),\
                                'import data - data_list.eh_tb_open')
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
            return
    

def import_table(frame, event):
    '''import_table(frame, event) --> None
    
    imports a table from the file given by a file dialog box
    '''
    dlg = wx.FileDialog(frame, message="Import script", defaultFile="",\
                    wildcard="Table File (*.tab)|*.tab|All files (*.*)|*.*",\
                         style=wx.FD_OPEN | wx.FD_CHANGE_DIR
                       )
    if dlg.ShowModal() == wx.ID_OK:
        try:
            frame.model.import_table(dlg.GetPath())
        except modellib.IOError as e:
            ShowModelErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText(\
                    'Error when importing script', 1)
            dlg.Destroy()
            return
        except Exception as e:
            ShowErrorDialog(frame, str(e),\
                                'import script - model.import_script')
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
            dlg.Destroy()
            return
            
    dlg.Destroy()
    # Post event to tell that the model has cahnged
    _post_new_model_event(frame, frame.model)
    frame.main_frame_statusbar.SetStatusText('Table imported from file', 1)

def parameter_value_changed(frame, event):
    """ Event handler for when a value of a parameter in the grid has been updated.

    :param frame:
    :param event:
    :return:
    """
    frame.simulation_queue_counter += 1
    if frame.main_frame_menubar.mb_fit_autosim.IsChecked() and not frame.flag_simulating:
        _thread.start_new_thread(simulation_loop, (frame,))

def simulation_loop(frame):
    """ Simulation loop for threading to increase the speed of the interactive simulations
    :param frame:
    :return:
    """
    frame.flag_simulating = True
    while frame.simulation_queue_counter > 0:
        do_simulation(frame)
        time.sleep(0.1)
        frame.simulation_queue_counter = min(1, frame.simulation_queue_counter - 1)
    frame.flag_simulating = False


def evaluate(frame, event):
    '''evaluate(frame, event) --> None
    
    Envent handler for only evaluating the Sim function - no recompiling
    '''
    frame.flag_simulating = True
    frame.main_frame_statusbar.SetStatusText('Simulating...', 1)
    # Compile is not necessary when using simualate...
    #frame.model.compile_script()
    try:
        frame.model.simulate(compile = False)
    except modellib.GenericError as e:
        ShowModelErrorDialog(frame, str(e))
        frame.main_frame_statusbar.SetStatusText('Error in simulation', 1)
    except Exception as e:
        outp = StringIO()
        traceback.print_exc(200, outp)
        val = outp.getvalue()
        outp.close()
        ShowErrorDialog(frame, val)
        frame.main_frame_statusbar.SetStatusText('Fatal Error - simulate', 1)
    else:
        _post_sim_plot_event(frame, frame.model, 'Simulation')
        frame.main_frame_statusbar.SetStatusText('Simulation Sucessful', 1)
        frame.plugin_control.OnSimulate(None)
    frame.flag_simulating = False
        
    
def simulate(frame, event):
    '''
    simulate(frame, event) --> None
    
    Event handler for simulation.
    '''
    frame.flag_simulating = True
    do_simulation(frame)
    set_possible_parameters_in_grid(frame)
    frame.flag_simulating = False


def do_simulation(frame):
    # Just a debugging output...
    # print frame.script_editor.GetText()

    frame.main_frame_statusbar.SetStatusText('Simulating...', 1)
    frame.model.set_script(frame.script_editor.GetText())
    # Compile is not necessary when using simualate...
    #frame.model.compile_script()
    try:
        frame.model.simulate()
    except modellib.GenericError as e:
        ShowModelErrorDialog(frame, str(e))
        frame.main_frame_statusbar.SetStatusText('Error in simulation', 1)
    except Exception as e:
        outp = StringIO()
        traceback.print_exc(200, outp)
        val = outp.getvalue()
        outp.close()
        ShowErrorDialog(frame, val)
        frame.main_frame_statusbar.SetStatusText('Fatal Error - simulate', 1)
    else:
        _post_sim_plot_event(frame, frame.model, 'Simulation')
        frame.plugin_control.OnSimulate(None)
        frame.main_frame_statusbar.SetStatusText('Simulation Sucessful', 1)
    
def set_possible_parameters_in_grid(frame):
    # Now we should find the parameters that we can use to
    # in the grid
    try:
        pardict = frame.model.get_possible_parameters()
    except Exception as e:
        outp = StringIO()
        traceback.print_exc(200, outp)
        val = outp.getvalue()
        outp.close()
        #ShowErrorDialog(frame, val)
        ShowErrorDialog(frame, val,\
            'simulate - model.get_possible_parameters')
        frame.main_frame_statusbar.SetStatusText('Fatal Error', 0)
        return
    
    try:
        frame.paramter_grid.SetParameterSelections(pardict)
    except Exception as e:
        ShowErrorDialog(frame, str(e),\
            'simulate - parameter_grid.SetParameterSelection')
        frame.main_frame_statusbar.SetStatusText('Fatal Error', 0)
        return
    # Set the function for which the parameter can be evaluated with
    frame.paramter_grid.SetEvalFunc(frame.model.eval_in_model)
    
def start_fit(frame, event):
    '''start_fit(frame, event) --> None
    
    Event handler to start fitting
    '''
    if frame.model.compiled:
        try:
            frame.solver_control.StartFit()
        except modellib.GenericError as e:
            ShowModelErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText('Error in fitting', 1)
        except Exception as e:
            ShowErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
        else:
            frame.main_frame_statusbar.SetStatusText('Fitting starting ...', 1)
    else:
        ShowNotificationDialog(frame, 'The script is not compiled, do a'\
        ' simulation before you start fitting.')
    
def stop_fit(frame, event):
    '''stop_fit(frame, event) --> None
    
    Event handler to stop the fitting routine
    '''
    frame.solver_control.StopFit()
    
def resume_fit(frame, event):
    '''resume_fit(frame, event) --> None
    
    Event handler to resume the fitting routine. No initilization.
    '''
    if frame.model.compiled:
        try:
            frame.solver_control.ResumeFit()
        except modellib.GenericError as e:
            ShowModelErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText('Error in fitting', 1)
        except Exception as e:
            ShowErrorDialog(frame, str(e))
            frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
        else:
            frame.main_frame_statusbar.SetStatusText('Fitting starting ...',1)
    else:
        ShowNotificationDialog(frame, 'The script is not compiled, do a'\
        ' simulation before you start fitting.')
    
    
def calculate_error_bars(frame, evt):
    '''calculate_error_bars(frame, evt) --> None
    
    callback to calculate the error bars on the data.
    '''
    try:
        error_values = frame.solver_control.CalcErrorBars()
    except solvergui.ErrorBarError as e:
        ShowNotificationDialog(frame, str(e))
    except Exception as e:
        ShowErrorDialog(frame, str(e), 'solvergui - CalcErrorBars')
        frame.main_frame_statusbar.SetStatusText('Fatal Error', 1)
    else:
        frame.model.parameters.set_error_pars(error_values)
        frame.paramter_grid.SetParameters(frame.model.parameters)
        frame.main_frame_statusbar.SetStatusText('Errorbars calculated', 1)

def scan_parameter(frame, row):
    ''' scan_parameter(frame, row) --> None
    
    Scans the parameter in row row [int] from max to min in the number
    of steps given by dialog input.
    '''
    if not frame.model.is_compiled():
        ShowNotificationDialog(frame, 'Please conduct a simulation before' +\
        ' scanning a parameter. The script needs to be compiled.')
        return
    
    dlg = wx.NumberEntryDialog(frame,\
                'Input the number of evaluation points for the scan',\
                'Steps', '', 50, 2, 1000)
    if dlg.ShowModal() ==wx.ID_OK:        
        frame.main_frame_statusbar.SetStatusText('Scanning parameter', 1)
        try:
            x, y = frame.solver_control.ScanParameter(row, dlg.GetValue())
            fs, pars = frame.model.get_sim_pars()
            bestx = frame.model.parameters.get_data()[row][1]
            besty = frame.model.fom
            
            frame.plot_fomscan.SetPlottype('scan')
            frame.plot_fomscan.Plot((x, y, bestx, besty,\
                        frame.solver_control.fom_error_bars_level),
                        frame.model.parameters.get_names()[row],
                        'FOM')
            frame.sep_plot_notebook.SetSelection(3)
        except Exception as e:
            outp = StringIO()
            traceback.print_exc(200, outp)
            val = outp.getvalue()
            outp.close()
            ShowErrorDialog(frame, val)
            frame.main_frame_statusbar.SetStatusText('Fatal Error - scan fom', 1)
        else:
            frame.main_frame_statusbar.SetStatusText('Scanning finished', 1)
        
    dlg.Destroy()
    
def project_fom_parameter(frame, row):
    '''project_fom_parameter(frame, row) --> None
    
    Plots the project fom given by the row row [int]
    '''
    if not frame.solver_control.IsFitted():
        ShowNotificationDialog(frame, 'Please conduct a fit before' +
        ' scanning a parameter. The script needs to be compiled and foms have'
         + ' to be collected.')
        return

    frame.main_frame_statusbar.SetStatusText('Trying to project fom', 1)
    try:
        x, y = frame.solver_control.ProjectEvals(row)
        if len(x) == 0 or len(y) == 0:
            ShowNotificationDialog(frame, 'Please conduct a fit before' +
                                   ' projecting a parameter. The script needs to be compiled and foms have'
                                   + ' to be collected.')
            return
        elif frame.model.fom is None or np.isnan(frame.model.fom):
            ShowNotificationDialog(frame, 'The model must be simulated (FOM is not a valid number)')
            return
        fs, pars = frame.model.get_sim_pars()
        bestx = pars[row]
        besty = frame.model.fom
        frame.plot_fomscan.SetPlottype('project')
        frame.plot_fomscan.Plot((x, y, bestx, besty,
                        frame.solver_control.fom_error_bars_level),
                        frame.model.parameters.get_names()[row],
                        'FOM')
        frame.sep_plot_notebook.SetSelection(3)
    except Exception as e:
        outp = StringIO()
        traceback.print_exc(200, outp)
        val = outp.getvalue()
        outp.close()
        ShowErrorDialog(frame, val)
        frame.main_frame_statusbar.SetStatusText('Fatal Error - project fom', 1)
    else:    
        frame.main_frame_statusbar.SetStatusText('Projected fom plotted', 1)
    
def on_optimizer_settings(frame, event):
    '''on_optimizer_settings(self, event) --> None
    
    Show the settings dialog for the optimizer
    '''
    #try:
    frame.solver_control.ParametersDialog(frame)
    #except Exception, e:
    #    raise e
    
def on_data_loader_settings(frame, event):
    '''on_data_loader_settings(frame, event) --> None
    
    Show the data_loader settings dialog. Allow the user to change the 
    data loader.
    '''
    frame.data_list.DataLoaderSettingsDialog()
    
def quit(frame, event):
    '''quit(frame, event) --> None
    
    Quit the program 
    '''
    # Check so the model is saved before quitting
    if not frame.model.saved:
        ans = ShowQuestionDialog(frame, 'If you continue any changes in' 
                                 ' your model will not be saved.', 
                                 'Model not saved')
        if not ans:
            return

    import appdirs
    config_path=appdirs.user_data_dir('GenX3', 'ArturGlavic')+'/'
    hsize, vsize=frame.GetSize()
    frame.config.default_set('gui', 'hsize', hsize)
    frame.config.default_set('gui', 'vsize', vsize)
    frame.config.default_set('gui', 'vsplit', frame.ver_splitter.GetSashPosition())
    frame.config.default_set('gui', 'hsplit', frame.hor_splitter.GetSashPosition())
    frame.config.default_set('gui', 'psplit', frame.plot_splitter.GetSashPosition())
    frame.config.write_default(config_path+'genx.conf')

    # app=frame.parent
    frame.findreplace_dlg.Destroy()
    frame.findreplace_dlg=None
    event.Skip()
    frame.Destroy()
    # app.ExitMainLoop()

    
def status_text(frame, event):
    '''status_text(frame, event) --> None
    Print a status text in the window. event should have a string
    member text. This will display the message in the status bar.
    '''
    frame.main_frame_statusbar.SetStatusText(event.text, 1)
    
def fom_value(frame, event):
    '''fom_value(frame, event) --> None
    
    Callback to update the fom_value displayed by the gui
    '''
    fom_value = event.model.fom
    fom_name =  event.model.fom_func.__name__
    if fom_value:
        frame.main_frame_fom_text.SetLabel('        FOM %s: %.4e'%(fom_name,fom_value))
    else:
        frame.main_frame_fom_text.SetLabel('        FOM %s: None'%(fom_name))
def point_pick(frame, event):
    '''point_pick(frame, event) --> None
    Callback for the picking of a data point in a plotting window.
    This will display the message in the status bar.
    '''
    frame.main_frame_statusbar.SetStatusText(event.text, 2)

def on_zoom_check(frame, event):
    '''on_zoom_toolbar(event) --> none
    
    Takes care of clicks on the toolbar zoom button and the menu item zoom.
    '''
    if event.GetId()==10009:
        zoom_state=frame.main_frame_toolbar.GetToolState(10009)
        frame.main_frame_menubar.mb_view_zoom.Check(zoom_state)
    else:
        zoom_state=frame.main_frame_menubar.mb_view_zoom.IsChecked()
        frame.main_frame_toolbar.ToggleTool(10009, zoom_state)

    # Synchronize all plots with zoom state
    pages = get_pages(frame)
    for page in pages:
        page.SetZoom(zoom_state)

def zoomall(frame, event):
    '''zoomall(self, event) --> None
    
    Zoom out and show all data points
    '''
    sel = frame.sep_plot_notebook.GetSelection()
    pages = get_pages(frame)
    if sel < len(pages):
        tmp = pages[sel].GetAutoScale()
        pages[sel].SetAutoScale(True)
        pages[sel].AutoScale()
        pages[sel].SetAutoScale(tmp)
        pages[sel].AutoScale()
        
def set_yscale(frame, type):
    '''set_yscale(frame, type) --> None
    
    Set the y-scale of the current plot. type should be linear or log, strings. 
    '''
    sel = frame.sep_plot_notebook.GetSelection()
    pages = get_pages(frame)
    if sel < len(pages):
        pages[sel].SetYScale(type)

def set_xscale(frame, type):
    '''set_xscale(frame, type) --> None

    Set the x-scale of the current plot. type should be linear or log, strings.
    '''
    sel = frame.sep_plot_notebook.GetSelection()
    pages = get_pages(frame)
    if sel < len(pages):
        pages[sel].SetXScale(type)


def on_grid_slider_check(frame, event):
    """Change the state of the grid value input, either as slider or as a number.
    :param frame:
    :param event:
    :return:
    """
    frame.paramter_grid.SetValueEditorSlider(frame.main_frame_menubar.mb_view_grid_slider.IsChecked())
    #print frame.paramter_grid.get_toggle_slider_tool_state()
    frame.paramter_grid.toggle_slider_tool(frame.main_frame_menubar.mb_view_grid_slider.IsChecked())
    frame.paramter_grid.Refresh()


def on_autoscale(frame, event):
    '''on_autoscale(frame, event) --> None
    
    Toggles the autoscale of the current plot.
    '''
    sel = frame.sep_plot_notebook.GetSelection()
    pages = get_pages(frame)
    if sel < len(pages):
        pages[sel].SetAutoScale(not pages[sel].GetAutoScale())
    
                
def plot_settings_changed(frame, event):
    '''zoom_changed(frame, event) --> None
    
    Callback for the settings change event for the current plot
     - change the toggle for the zoom icon and change the menu items.
    '''
    frame.main_frame_toolbar.ToggleTool(10009, event.zoomstate)
    frame.main_frame_menubar.mb_view_zoom.Check(event.zoomstate)
    if event.yscale == 'log':
        frame.main_frame_menubar.mb_view_yscale_log.Check(True)
    elif event.yscale == 'linear':
        frame.main_frame_menubar.mb_view_yscale_lin.Check(True)
    if event.xscale == 'log':
        frame.main_frame_menubar.mb_view_xscale_log.Check(True)
    elif event.xscale == 'linear':
        frame.main_frame_menubar.mb_view_xscale_lin.Check(True)
    frame.main_frame_menubar.mb_view_autoscale.Check(event.autoscale)
    
def plot_page_changed(frame, event):
    '''plot_page_changed(frame, event) --> None
        
    Callback for page change in plot notebook. Changes the state of
    the zoom toggle button.
    '''
    sel = event.GetSelection()
    pages = get_pages(frame)
    if sel < len(pages):
        zoom_state = pages[sel].GetZoom()        
        # Set the zoom button to the correct value
        frame.main_frame_toolbar.ToggleTool(10009, zoom_state)
        frame.main_frame_menubar.mb_view_zoom.Check(zoom_state)
        
        yscale = pages[sel].GetYScale()
        if yscale == 'log':
            frame.main_frame_menubar.mb_view_yscale_log.Check(True)
        elif yscale == 'linear':
            frame.main_frame_menubar.mb_view_yscale_lin.Check(True)
        xscale = pages[sel].GetXScale()
        if xscale == 'log':
            frame.main_frame_menubar.mb_view_yscale_log.Check(True)
        elif xscale == 'linear':
            frame.main_frame_menubar.mb_view_yscale_lin.Check(True)

def print_plot(frame, event):
    '''print_plot(frame, event) --> None
    
    prints the current plot in the plot notebook.
    '''
    sel = frame.sep_plot_notebook.GetSelection()
    pages = get_pages(frame)
    if sel < len(pages):
        pages[sel].Print()
        
def print_preview_plot(frame, event):
    '''print_preview_plot(frame, event) --> None
    
    prints a preview of the current plot int the plot notebook.
    '''
    sel = frame.sep_plot_notebook.GetSelection()
    pages = get_pages(frame)
    if sel < len(pages):
        pages[sel].PrintPreview()
        
def print_parameter_grid(frame, event):
    ''' print_parameter_grid(frame, event) --> None
    
    Prints the table of parameters that have been fitted.
    '''
    frame.paramter_grid.Print()
    #genxprint.TablePanel(frame)
   
def print_preview_parameter_grid(frame, event):
    ''' print_parameter_grid(frame, event) --> None
    
    Prints the table of parameters that have been fitted.
    '''
    frame.paramter_grid.PrintPreview()

    
def copy_graph(frame, event):
    '''copy_graph(self, event) --> None
    
    Callback that copies the current graph in the plot notebook to
    the clipboard.
    '''
    sel = frame.sep_plot_notebook.GetSelection()
    pages = get_pages(frame)
    if sel < len(pages):
        pages[sel].CopyToClipboard()
        
def copy_table(frame, event):
    '''copy_table(frame, event) --> None
    
    Copies the table as ascii text to the clipboard
    '''
    ascii_table = frame.paramter_grid.table.pars.get_ascii_output()
    text_table=wx.TextDataObject(ascii_table)
    if wx.TheClipboard.Open():
        wx.TheClipboard.SetData(text_table)
        wx.TheClipboard.Close()
        
def copy_sim(frame, event):
    '''copy_sim(frame, event) --> None
    
    Copies the simulation and the data to the clipboard. Note that this
    copies ALL data.
    '''
    text_string = frame.model.get_data_as_asciitable()
    text = wx.TextDataObject(text_string)
    if wx.TheClipboard.Open():
        wx.TheClipboard.SetData(text)
        wx.TheClipboard.Close()

def on_findreplace(frame, event):
    '''Show the find and replace dialog box.
    '''
    frame.findreplace_dlg.Show(True)

def on_find_event(frame, event):
    '''callback for find events - coupled to the script
    '''
    evtype = event.GetEventType()
    def find():
        find_str = event.GetFindString()
        ##print frame.findreplace_data.GetFlags()
        flags = event.GetFlags()
        if flags & 1:
            ##print "Searching down"
            pos = frame.script_editor.SearchNext(flags, find_str)
        else:
            ##print "Searching up"
            pos = frame.script_editor.SearchPrev(flags, find_str)
        if pos == -1:
            frame.main_frame_statusbar.SetStatusText(\
                'Could not find text %s'%find_str, 1)
        return pos

    def replace():
        replace_str = event.GetReplaceString()
        frame.script_editor.ReplaceSelection(replace_str)
        
    # Deal with the different cases
    if evtype == wx.wxEVT_COMMAND_FIND:
        frame.script_editor.SearchAnchor()
        find()
        
    elif evtype == wx.wxEVT_COMMAND_FIND_NEXT:
        pnew = frame.script_editor.GetSelectionEnd()
        ##print pnew
        frame.script_editor.GotoPos(pnew)
        frame.script_editor.SetAnchor(pnew)
        frame.script_editor.SearchAnchor()
        ##print 'Finding next'
        find()
        
    elif evtype == wx.wxEVT_COMMAND_FIND_REPLACE:
        #print 'find and replace'
        # If we do not have found text already
        # or if we have marked other text by mistake...
        if frame.script_editor.GetSelectedText() != \
               event.GetFindString():
            find()
        # We already have found and marked text that we should
        # replace
        else:
            frame.script_editor.ReplaceSelection(\
                event.GetReplaceString())
            # Find a new text to replace
            find()
    elif evtype == wx.wxEVT_COMMAND_FIND_REPLACE_ALL:
        #print 'find and replace all'
        if frame.script_editor.GetSelectedText() != \
               event.GetFindString():
            pos = find()
        i = 0
        while pos != -1:
            frame.script_editor.ReplaceSelection(\
                event.GetReplaceString())
            i += 1
            pos = find()
        frame.main_frame_statusbar.SetStatusText(\
                'Replaces %d occurancies of  %s'%(i,\
                 event.GetFindString()), 1)
        
    ##else:
    ##    ShowErrorDialog(frame, 'Faulty event supplied in find and'\
    ##                    ' repalce functionallity', 'on_find_event')
    # This will scroll the editor to the right position so we can see
    # the text
    frame.script_editor.EnsureCaretVisible()
        
def change_data_grid_view(frame, event):
    '''change_data_grid_view(frame, event) --> None
    
    change the data displayed in the grid...
    '''
    #print event.GetSelection()
    dataset = frame.model.data[event.GetSelection()]
    rows = frame.data_grid.GetNumberRows()
    new_rows = max(len(dataset.x), len(dataset.y),\
                    len(dataset.x_raw), len(dataset.y_raw))
    frame.data_grid.DeleteRows(numRows = rows)
    frame.data_grid.AppendRows(new_rows)
    [[frame.data_grid.SetCellValue(row, col, '-') for col in range(6)]\
        for row in range(new_rows)]
    [frame.data_grid.SetCellValue(row, 0, '%.3e'%dataset.x_raw[row])\
        for row in range(len(dataset.x_raw))]
    [frame.data_grid.SetCellValue(row, 1, '%.3e'%dataset.y_raw[row])\
        for row in range(len(dataset.y_raw))]
    [frame.data_grid.SetCellValue(row, 2, '%.3e'%dataset.error_raw[row])\
        for row in range(len(dataset.error_raw))]
    [frame.data_grid.SetCellValue(row, 3, '%.3e'%dataset.x[row])\
        for row in range(len(dataset.x))]
    [frame.data_grid.SetCellValue(row, 4, '%.3e'%dataset.y[row])\
        for row in range(len(dataset.y))]
    [frame.data_grid.SetCellValue(row, 5, '%.3e'%dataset.error[row])\
        for row in range(len(dataset.error))]
        
def update_data_grid_choice(frame, event):
    '''update_data_grid_choice(frame, event) --> None
    
    Updates the choices of the grids to display from the data.
    '''
    data = event.GetData()
    names = [data_set.name for data_set in data]
    #frame.data_grid_choice.SetItems(names)
    frame.data_grid_choice.Clear()
    frame.data_grid_choice.AppendItems(names)
    event.Skip()
    
def update_data(frame, event):
    '''update_data(frame, event) --> None
    
    callback for updating data, right now in the plugins
    '''
    frame.plugin_control.OnDataChanged(event)

def activate_cuda(frame, event):
    dlg = wx.ProgressDialog(parent=frame,
                            maximum=3,
                            message="Compiling CUDA GPU computing functions with Numba",
                            title='Activating CUDA...')
    dlg.Show()

    dlg.Update(1)
    from genx.models.lib import paratt_cuda
    dlg.Update(2)
    from genx.models.lib import neutron_cuda
    dlg.Update(3)
    from models.lib import paratt, neutron_refl
    paratt.Refl=paratt_cuda.Refl
    paratt.ReflQ=paratt_cuda.ReflQ
    paratt.Refl_nvary2=paratt_cuda.Refl_nvary2
    neutron_refl.Refl=neutron_cuda.Refl
    from genx.models.lib import paratt, neutron_refl
    paratt.Refl=paratt_cuda.Refl
    paratt.ReflQ=paratt_cuda.ReflQ
    paratt.Refl_nvary2=paratt_cuda.Refl_nvary2
    neutron_refl.Refl=neutron_cuda.Refl

    # cuda does not work with multiple processes as those do not use the fuctions forced above
    frame.solver_control.optimizer.use_parallel_processing=False
    frame.solver_control.WriteConfig()

    dlg.Destroy()


def deactivate_cuda(frame, event):
    from genx.models.lib import paratt_numba, neutron_numba
    from models.lib import paratt, neutron_refl
    paratt.Refl=paratt_numba.Refl
    paratt.ReflQ=paratt_numba.ReflQ
    paratt.Refl_nvary2=paratt_numba.Refl_nvary2
    neutron_refl.Refl=neutron_numba.Refl
    from genx.models.lib import paratt, neutron_refl
    paratt.Refl=paratt_numba.Refl
    paratt.ReflQ=paratt_numba.ReflQ
    paratt.Refl_nvary2=paratt_numba.Refl_nvary2
    neutron_refl.Refl=neutron_numba.Refl


def fom_help(frame, event):
    '''Show a help dialog for information about the different fom.
    '''
    dlg = help.PluginHelpDialog(frame, 'fom_funcs', title="FOM functions help")
    dlg.Show()

def models_help(frame, event):
    '''models_help(frame, event) --> None
    
    Show a help dialog for information about the different models.
    '''
    dlg = help.PluginHelpDialog(frame,'models', title="Models help")
    dlg.Show()
    
def plugins_help(frame, event):
    '''plugins_help(frame, event) --> None
    
    Show a help dialog for information about the different plugins.
    '''
    dlg = help.PluginHelpDialog(frame, 'plugins.add_ons', title="Plugins help")
    dlg.Show()

def data_loaders_help(frame, event):
    '''data_loaders_help(frame, event) --> None
    
    Show a help dialog for information about the different data_loaders.
    '''
    dlg = help.PluginHelpDialog(frame,'plugins.data_loaders', title="Data loaders help")
    dlg.Show()    
    
def show_manual(frame, event):
    '''show_manual(frame, event) --> None
    
    Callback to show the manual
    '''
    #ShowNotificationDialog(frame, 'There is no manual yet!')
    webbrowser.open_new(manual_url)
    
def show_homepage(frame, event):
    '''show_homepage(frame, event) --> None
    
    Callback to show the homepage
    '''
    webbrowser.open_new(homepage_url)

def show_about_box(frame, event):
    '''show_about_box(frame, event) --> None
    
    Show an about box about GenX with some info...
    '''
    import numpy, scipy, matplotlib, platform
    useful=''
    try:
        import numba
        useful+='Numba: %s, '%numba.__version__
    except:
        pass
    try:
        import vtk
        useful+='VTK: %s, '%vtk.vtkVersion.GetVTKVersion()
    except:
        pass
    try:
        import dumps
        useful+='Dumps: %s, '%dumps.__version__
    except:
        pass

    info = wx.adv.AboutDialogInfo()
    info.Name = "GenX"
    info.Version = __version__
    info.Copyright = "(C) 2008 Matts Bjorck; 2020 Artur Glavic"
    info.Description = wordwrap(
        "GenX is a multipurpose refinement program using the differential "
        "evolution algorithm. It is developed  mainly for refining x-ray reflectivity "
        "and neutron reflectivity data."
        
        "\n\nThe versions of the mandatory libraries are:\n"
        "Python: %s, wxPython: %s, Numpy: %s, Scipy: %s, Matplotlib: %s"
        "\n\nThe non-mandatory but useful packages:\n%s"
        ""%(platform.python_version(), wx.__version__,
            numpy.__version__, scipy.__version__, \
            matplotlib.__version__, useful),
        500, wx.ClientDC(frame))
    info.WebSite = ("http:////genx.sourceforge.net", "GenX homepage")
    # No developers yet
    #info.Developers = []
    #head, tail = os.path.split(__file__)
    #license_text = file(head + '/LICENSE.txt','r').read()
    #license_text = file(_path + 'LICENSE.txt','r').read()
    #info.License = license_text#wordwrap(license_text, 500, wx.ClientDC(self))
    info.Licence = wordwrap('This program is free software: you can redistribute it and/or modify '
                            'it under the terms of the GNU General Public License as published by '
                            'the Free Software Foundation, either version 3 of the License, or '
                            '(at your option) any later version. '
                            '\n\nThis program is distributed in the hope that it will be useful, '
                            'but WITHOUT ANY WARRANTY; without even the implied warranty of '
                            'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the '
                            'GNU General Public License for more details. '
                            '\n\nYou should have received a copy of the GNU General Public License '
                            'along with this program.  If not, see <http://www.gnu.org/licenses/>. '
                            , 400, wx.ClientDC(frame))
    
    wx.adv.AboutBox(info)
        
#=============================================================================
# Custom events needed for updating and message parsing between the different
# modules.

class GenericModelEvent(wx.CommandEvent):
    '''
    Event class for a new model - for updating
    of the paramters, plots and script.
    '''
    
    def __init__(self,evt_type, id, model):
        wx.CommandEvent.__init__(self, evt_type, id)
        self.model = model
        self.description = ''
        
    def GetModel(self):
        return self.model
        
    def SetModel(self, model):
        self.model = model
        
    def SetDescription(self, desc):
        '''
        Set a string that describes the event that has occurred
        '''
        self.description = desc
        
# Generating an event type:
myEVT_NEW_MODEL = wx.NewEventType()
# Creating an event binder object
EVT_NEW_MODEL = wx.PyEventBinder(myEVT_NEW_MODEL)

def _post_new_model_event(parent, model, desc = ''):
    # Send an event that a new data set has been loaded
        evt = GenericModelEvent(myEVT_NEW_MODEL, parent.GetId(), model)
        evt.SetDescription(desc)
        # Process the event!
        parent.GetEventHandler().ProcessEvent(evt)


# Generating an event type:
myEVT_SIM_PLOT = wx.NewEventType()
# Creating an event binder object
EVT_SIM_PLOT = wx.PyEventBinder(myEVT_SIM_PLOT)

def _post_sim_plot_event(parent, model, desc = ''):
    # Send an event that a new data set ahs been loaded
        evt = GenericModelEvent(myEVT_SIM_PLOT, parent.GetId(), model)
        evt.SetDescription(desc)
        # Process the event!
        parent.GetEventHandler().ProcessEvent(evt)
        

#==============================================================================
## Functions for showing error dialogs

def ShowQuestionDialog(frame, message, title = 'Question?'):
    dlg = wx.MessageDialog(frame, message,
                               title,
                               wx.OK | wx.CANCEL | wx.OK_DEFAULT | wx.ICON_QUESTION
                               )
    result = dlg.ShowModal() == wx.ID_OK
    dlg.Destroy()
    return result

def ShowModelErrorDialog(frame, message):
    dlg = wx.MessageDialog(frame, message,
                               'Warning',
                               wx.OK | wx.ICON_WARNING
                               )
    dlg.ShowModal()
    dlg.Destroy()
    
def ShowNotificationDialog(frame, message):
    dlg = wx.MessageDialog(frame, message,
                               'Information',
                               wx.OK | wx.ICON_INFORMATION
                               )
    dlg.ShowModal()
    dlg.Destroy()
    
def ShowErrorDialog(frame, message, position = ''):
    if position != '':
        dlg = wx.MessageDialog(frame, message + '\n' + 'Position: ' + position,
                               'FATAL ERROR',
                               wx.OK | wx.ICON_ERROR
                               )
    else:
        dlg = wx.MessageDialog(frame, message,
                               'FATAL ERROR',
                               wx.OK | wx.ICON_ERROR
                               )
    dlg.ShowModal()
    dlg.Destroy()
#==============================================================================



'''
  Module for usefull decorators e.g. for logging function calls, input and output.
'''

import sys
import inspect
import logging
import threading
from time import time
from io import StringIO


#
# Help functions adopted from Michele Simionato's decorator module 
# http://www.phyast.pitt.edu/~micheles/python/decorator.zip
#

def getinfo(func):
    """
      Returns an info dictionary containing:
      - name (the name of the function : str)
      - argnames (the names of the arguments : list)
      - defaults (the values of the default arguments : tuple)
      - signature (the signature : str)
      - doc (the docstring : str)
      - module (the module name : str)
      - dict (the function __dict__ : str)
    """
    assert inspect.ismethod(func) or inspect.isfunction(func)
    regargs, varargs, varkwargs, defaults=inspect.getargspec(func)
    argnames=list(regargs)
    if varargs:
        argnames.append(varargs)
    if varkwargs:
        argnames.append(varkwargs)
    signature=inspect.formatargspec(regargs, varargs, varkwargs, defaults,
                                      formatvalue=lambda value: "")[1:-1]
    output=dict(name=func.__name__, argnames=argnames, signature=signature,
                defaults=func.__defaults__, doc=func.__doc__,
                module=func.__module__, dict=func.__dict__,
                globals=func.__globals__, closure=func.__closure__)
    return output

def update_wrapper(wrapper, wrapped, create=False):
    """
      An improvement over functools.update_wrapper. By default it works the
      same, but if the 'create' flag is set, generates a copy of the wrapper
      with the right signature and update the copy, not the original.
      Moreovoer, 'wrapped' can be a dictionary with keys 'name', 'doc', 'module',
      'dict', 'defaults'.
    """
    if isinstance(wrapped, dict):
        infodict=wrapped
    else: # assume wrapped is a function
        infodict=getinfo(wrapped)
    assert not '_wrapper_' in infodict["argnames"], \
           '"_wrapper_" is a reserved argument name!'
    if create: # create a brand new wrapper with the right signature
        src="lambda %(signature)s: _wrapper_(%(signature)s)"%infodict
        # import sys; print >> sys.stderr, src # for debugging purposes
        wrapper=eval(src, dict(_wrapper_=wrapper))        
    try:
        wrapper.__name__=infodict['name']
    except: # Python version < 2.4
        pass
    wrapper.__doc__=infodict['doc']
    wrapper.__module__=infodict['module']
    wrapper.__dict__.update(infodict['dict'])
    wrapper.__defaults__=infodict['defaults']
    return wrapper


# the real meat is here
def _decorator(caller, func):
    infodict=getinfo(func)
    argnames=infodict['argnames']
    assert not ('_call_' in argnames or '_func_' in argnames), \
           'You cannot use _call_ or _func_ as argument names!'
    src="lambda %(signature)s: _call_(_func_, %(signature)s)"%infodict
    dec_func=eval(src, dict(_func_=func, _call_=caller))
    return update_wrapper(dec_func, func)

def decorator(caller):
    """
    General purpose decorator factory: takes a caller function as
    input and returns a decorator with the same attributes.
    """
    return update_wrapper(lambda f : _decorator(caller, f), caller)


########################### Decorators for logging ##################################

def _nicetype(item):
  if 'array' in type(item).__name__:
    return type(item).__name__+("[%s]"%str(item.shape))
  elif type(item) in [list, tuple]:
    return type(item).__name__+('[%i]'%len(item))
  else:
    return type(item).__name__

class DecoratorLogger(logging.getLoggerClass()):
  '''
    A logger that makes sure the actual function definition filename, lineno and function name
    is used for logging.
  '''
  
  if sys.version_info[0:2]>=(3, 2): #sinfo was introduced in python 3.2
    def makeRecord(self, name, lvl, fn, lno, msg, args, exc_info, func=None, extra=None, sinfo=None):
      if extra is None:
        return logging.getLoggerClass().makeRecord(self, name, lvl, fn, lno,
                           msg, args, exc_info, func=func, extra=None, sinfo=sinfo)
      else:

        return logging.getLoggerClass().makeRecord(self, name, lvl, extra['name'], extra['lno'],
                           msg, args, exc_info, func=extra['func'], extra=None, sinfo=sinfo)
  else:
    def makeRecord(self, name, lvl, fn, lno, msg, args, exc_info, func=None, extra=None):
      if extra is None:
        return logging.getLoggerClass().makeRecord(self, name, lvl, fn, lno, 
                           msg, args, exc_info, func=func, extra=None)
      else:
        
        return logging.getLoggerClass().makeRecord(self, name, lvl, extra['name'], extra['lno'],
                           msg, args, exc_info, func=extra['func'], extra=None)

# only use the debug decorators in debugging runs.
old_class=logging.getLoggerClass()
logging.setLoggerClass(DecoratorLogger)
logger=logging.getLogger('deco')
logger.setLevel(logging.DEBUG)
null_handler=logging.StreamHandler(StringIO())
null_handler.setLevel(logging.CRITICAL)
logger.addHandler(null_handler)
logging.setLoggerClass(old_class)

def _logformat(msg, decname, func):
  if hasattr(func, 'im_func'):
    co=func.__func__.__code__
  else:
    co=func.__code__
  fname=co.co_filename
  lno=co.co_firstlineno
  logger.debug(msg, extra={'func': '@'+decname, 'name': fname, 'lno': lno})

@decorator
def log_call(func, *args, **kw):
  '''
    Decorator to log just the method call.
  '''
  if logging.root.getEffectiveLevel()>logging.DEBUG: return func(*args, **kw)
  infodict=getinfo(func)
  if len(infodict['argnames'])>0 and infodict['argnames'][0]=='self':
    _logformat('%s.%s.%s <%s>'%(infodict['module'],
                                   args[0].__class__.__name__, infodict['name'], repr(threading.current_thread())),
               'log_call', func)
  else:
    _logformat('%s.%s <%s>'%(infodict['module'], infodict['name'], repr(threading.current_thread())), 'log_call', func)
  return func(*args, **kw)

@decorator
def log_input(func, *args, **kw):
  '''
    Decorator to log a method call with input.
  '''
  if logging.root.getEffectiveLevel()>logging.DEBUG: return func(*args, **kw)
  infodict=getinfo(func)
  if hasattr(func, 'im_func'):
    logstr=' call %s.%s('%(args[0].__class__.__name__, infodict['name'])
    method=True
    info_len=len('%s.%s('%(args[0].__class__.__name__, infodict['name']))
  else:
    logstr='call %s('%(infodict['name'])
    method=False
    info_len=len('%s('%(infodict['name']))

  namelen=max([len(name)+3 for name in infodict['argnames']]+[len(name) for name in list(kw.keys())])
  for i, arg in enumerate(args):
    if i==0 and (method or infodict['name']=='__init__'):
      continue
    value=repr(arg)
    value_split=value.splitlines()
    if len(value_split)>5:
      value="\n".join(value_split[:2]+[' '*int(len(value_split[1])/2)+'...']+value_split[-2:])
    value.replace('\n', '\n'+' '*int(namelen+7))
    logstr+='\n'+('%% %is= %%s (%%s)'%(namelen+3))%(infodict['argnames'][i], value, _nicetype(arg))

  for key, arg in sorted(kw.items()):
    value=repr(arg)
    value_split=value.splitlines()
    if len(value_split)>5:
      value="\n".join(value_split[:2]+[' '*int(len(value_split[1])/2)+'...']+value_split[-2:])
    value.replace('\n', '\n'+' '*int(namelen+4))
    logstr+='\n'+('KW %% %ss= %%s (%%s)'%namelen)%(key, repr(value), _nicetype(arg))
  logstr+='\n)'
  logstr=logstr.replace('\n', '\n'+' '*int(info_len+10))
  logstr=logstr.replace('call ', 'call \n'+' '*10)

  _logformat(logstr, 'log_input', func)
  return func(*args, **kw)

@decorator
def log_output(func, *args, **kw):
  '''
    Decorator to log a method call with output. If combined with log_input
    the input is logged at the time before the call and the output after.
  '''
  if logging.root.getEffectiveLevel()>logging.DEBUG: return func(*args, **kw)
  output=func(*args, **kw)
  infodict=getinfo(func)
  if len(infodict['argnames'])>0 and infodict['argnames'][0]=='self':
    logstr='return from %s.%s'%(args[0].__class__.__name__, infodict['name'])
  else:
    logstr='return from %s'%(infodict['name'])
  value=repr(output)
  value_split=value.splitlines()
  if len(value_split)>5:
    value="\n".join(value_split[:2]+[' '*int(len(value_split[1])/2)+'...']+value_split[-2:])
  logstr+='\n-> %15s (%s)'%(value, _nicetype(output))
  logstr=logstr.replace('\n', '\n'+' '*44)
  _logformat(logstr, 'log_output', func)
  return output

@decorator
def log_both(func, *args, **kw):
  '''
    Decoratore to log a method call with input and output.
  '''
  if logging.root.getEffectiveLevel()>logging.DEBUG: return func(*args, **kw)
  infodict=getinfo(func)
  if hasattr(func, 'im_func'):
    logstr=' call %s.%s('%(args[0].__class__.__name__, infodict['name'])
    method=True
    info_len=len('%s.%s('%(args[0].__class__.__name__, infodict['name']))
  else:
    logstr='call %s('%(infodict['name'])
    method=False
    info_len=len('%s('%(infodict['name']))

  namelen=max([len(name)+3 for name in infodict['argnames']]+[len(name) for name in list(kw.keys())])
  for i, arg in enumerate(args):
    if i==0 and (method or infodict['name']=='__init__'):
      continue
    value=repr(arg)
    value_split=value.splitlines()
    if len(value_split)>5:
      value="\n".join(value_split[:2]+[' '*int(len(value_split[1])/2)+'...']+value_split[-2:])
    value.replace('\n', '\n'+' '*int(namelen+7))
    logstr+='\n'+('%% %is= %%s (%%s)'%(namelen+3))%(infodict['argnames'][i], value, _nicetype(arg))

  for key, arg in sorted(kw.items()):
    value=repr(arg)
    value_split=value.splitlines()
    if len(value_split)>5:
      value="\n".join(value_split[:2]+[' '*int(len(value_split[1])/2)+'...']+value_split[-2:])
    value.replace('\n', '\n'+' '*int(namelen+4))
    logstr+='\n'+('KW %% %ss= %%s (%%s)'%namelen)%(key, repr(value), _nicetype(arg))
  logstr+='\n)'
  logstr=logstr.replace('\n', '\n'+' '*int(info_len+10))
  logstr=logstr.replace('call ', 'call \n'+' '*10)

  _logformat(logstr, 'log_input', func)
  # call the function
  output=func(*args, **kw)
  if len(infodict['argnames'])>0 and infodict['argnames'][0]=='self':
    logstr='return from %s.%s'%(args[0].__class__.__name__, infodict['name'])
  else:
    logstr='return from %s'%(infodict['name'])
  value=repr(output)
  value_split=value.splitlines()
  if len(value_split)>5:
    value="\n".join(value_split[:2]+[' '*int(len(value_split[1])/2)+'...']+value_split[-2:])
  logstr+='\n-> %s (%s)'%(value, _nicetype(output))
  logstr=logstr.replace('\n', '\n'+' '*int(info_len+10))
  _logformat(logstr, 'log_output', func)
  return output

timings={}
@decorator
def time_call(func, *args, **kw):
  '''
    Decorator to log just the method call.
  '''
  if logging.root.getEffectiveLevel()>logging.DEBUG: return func(*args, **kw)
  name=func.__name__
  start=time()
  output=func(*args, **kw)
  runtime=time()-start
  if not name in timings:
    timings[name]=(0., 0.)
  avg, calls=timings[name]
  timings[name]=((avg*calls+runtime)/(calls+1), calls+1)
  return output

########################## General decorators ###############################

class check_input(object):
  '''
    Decorator checking the input to a function.
    
    Usage:
    @check_input([type1, type2, ...], try_convert=True)
    def your_function(var1, var2, ...):
        ... function text ...
  '''
  try_convert=True
  types=[]
  
  def __init__(self, types, try_convert=True):
    self.types=types
    self.try_convert=try_convert
  
  def __call__(self, func):
    infodict=getinfo(func)
    argnames=infodict['argnames']
    assert not ('_call_' in argnames or '_func_' in argnames), \
           'You cannot use _call_ or _func_ as argument names!'
    src="def dec_func(%(signature)s):"%infodict
    for i, typei in enumerate(self.types):
      src+='\n  if type(%s).__name__!="%s":'%(argnames[i], typei.__name__)
      if self.try_convert:
        src+='\n    try:'
        src+='\n      %s=%s(%s)'%(argnames[i], typei.__name__, argnames[i])
        src+='\n    except:'
        src+='\n      raise ValueError("type of %s is not %s")'%(argnames[i], typei.__name__)
      else:
        src+='\n    raise ValueError("type of %s is not %s")'%(argnames[i], typei.__name__)
    src+='\n  return _func_(%(signature)s)'%infodict
    exec_dict=dict(_func_=func, _call_=self.__call__)
    exec(src, exec_dict)
    return update_wrapper(exec_dict['dec_func'], func)



'''framework.py

Library that implements a template (Template) class for classes that
loads data into GenX. Also included here is a DataLoaderController which
takes care of the use of the DataLoaders. 
'''
import wx, os, io, traceback

from .utils import PluginHandler
from genx.gui_logging import iprint

head, tail = os.path.split(__file__)
# Look only after the file name and not the ending since
# the file ending can be pyc if compiled... 
__FILENAME__ = tail.split('.')[0]
# This assumes that plugin is under the current dir may need 
# changing
__MODULE_DIR__ = head
if __MODULE_DIR__ != '/':
    __MODULE_DIR__ += '/'


class Template:
    def __init__(self, parent):
        self.parent = parent
        # This is made for the virtual datalist controller...
        self.data = self.parent.data_cont.get_data()
        self.Register()
        
    def Register(self):
        '''Register(self) --> None
        
        Register the data loader with the parent
        '''
        self.parent.data_loader = self
        
    def SetData(self, data):
        '''SetData(self, data) --> None
        
        Sets the data connection to the plugin.
        '''
        self.data = data
        
    def UpdateDataList(self):
        '''UpdateDataList(self) --> None
        
        Forces the data list to update. This is only necessary if new
        data sets have been added when the data has been loaded
        '''
        # Just a force update of the data_list
        self.parent.SetItemCount(\
            self.parent.data_cont.get_count())
        # Updating the imagelist as well
        self.parent._UpdateImageList()
        
    def SetStatusText(self, text):
        '''SetStatusText(self, text) --> None
        
        Set a status text in the main frame for user information 
        '''
        self.parent.SetStatusText(text)
    
    def LoadDataFile(self, selected_items):
        '''LoadDataFile(self, seleceted_items) --> None
        
        Selected items is the selcted items in the items in the current DataList
        into which data from file(s) should be loaded. Note that the default
        implementation only allows the loading of a single file! 
        Overriding this function in subclasses can of course change this 
        behaviour. This function calls the LoadData function which implements 
        the io function by it self. The LoadData has to be overloaded in
        order to have a working plugin.
        '''
        n_selected = len(selected_items)
        if n_selected == 1:
            dlg = wx.FileDialog(self.parent, message = "Choose your Datafile"
                    , defaultFile = "", wildcard = "All files (*.*)|*.*"
                    , style=wx.FD_OPEN | wx.FD_CHANGE_DIR)
                    
            if dlg.ShowModal() == wx.ID_OK:
                self.LoadData(selected_items[0], dlg.GetPath())
                return True
            dlg.Destroy()
        else:
            if n_selected > 1:
                dlg = wx.MessageDialog(self.parent,\
                    'Please select only one dataset'\
                    , caption = 'Too many selections'
                    , style = wx.OK|wx.ICON_INFORMATION)
            else:
                dlg = wx.MessageDialog(self.parent, 'Please select a dataset'
                    , caption = 'No active dataset'
                    , style = wx.OK|wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
        
        return False
    
    def LoadData(self, data_item, file_path):
        '''LoadData(self, data_item, file_path) --> None
        
        This file should load a single data file into data object of
        the model. Please overide this function. It is called by the 
        LoadFile function.
        '''
        pass
        
    def SettingsDialog(self):
        '''SettingsDialog(self) --> None
        
        This function should - if necessary implement a dialog box
        that allows the user set import settings for example.
        '''
        pass
    
    def SendUpdateDataEvent(self):
        '''SendUpdateDataEvent(self) --> None
        
        Sends an event that new data has been loaded and 
        things such as plotting should be updated.
        '''
        self.parent._UpdateData('New data added')
        
    def Remove(self):
        '''Remove(self) --> None
        
        Removes the plugin from knowledge of the parent.
        '''
        self.parent.data_loader = None
        
class PluginController:
    def __init__(self, parent):
        self.plugin_handler = PluginHandler(parent, __MODULE_DIR__ \
                            , 'data_loaders')
        self.parent = parent
        self.plugin_handler.load_plugin('default')
        
    def load_default(self):
        try:
            plugin_name = self.parent.config.get('data handling', 'data loader')
            self.LoadPlugin(plugin_name)
        except Exception as S:
            iprint('Could not locate the data loader parameter or the data loader. Error:')
            iprint(S.__str__())
            iprint('Proceeding with laoding the default data loader.')
            self.LoadPlugin('default')
        
    def LoadPlugin(self, plugin):
        '''LoadPlugin(self, plugin) --> None
        
        Loads a data handler note that there is no UnLoad function
        since only one DataHandler can be plugged in at a time.
        '''
        # Unload the plugins
        names = self.plugin_handler.loaded_plugins.copy()
        try:
            [self.plugin_handler.unload_plugin(pl) for pl in names]
            self.parent.SetStatusText('Unloaded data loader %s'%list(names.keys())[0])
        except:
            outp = io.StringIO()
            traceback.print_exc(200, outp)
            tbtext = outp.getvalue()
            outp.close()
            ShowErrorDialog(self.parent, 'Can NOT unload plugin object'+ \
                list(names.keys())[0] + '\nPython traceback below:\n\n' + tbtext)
        try:
            self.plugin_handler.load_plugin(plugin)
            self.parent.SetStatusText('Loaded data loader: %s'%plugin)
        except:
            outp = io.StringIO()
            traceback.print_exc(200, outp)
            tbtext = outp.getvalue()
            outp.close()
            ShowErrorDialog(self.parent, 'Can NOT load plugin ' + plugin\
             + '\nPython traceback below:\n\n' + tbtext)
        
    def ShowDialog(self):
        '''ShowDialog(self) --> None
        
        Shows a dialog boc for the user to choose a data loader.
        '''
        cur_plugin = list(self.plugin_handler.loaded_plugins.keys())[0]
        plugin_list = self.plugin_handler.get_possible_plugins()
        dlg = PluginDialog(self.parent, plugin_list, cur_plugin,\
                                self.LoadPlugin)
        dlg.ShowModal()
        dlg.Destroy()
        
        
class PluginDialog(wx.Dialog):
    def __init__(self, parent, plugin_list, current_plugin,\
            load_plugin_func = None):
        wx.Dialog.__init__(self, parent, -1, 'Choose a data loader')
        
        self.load_plugin_func = load_plugin_func
        
        choice_sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer = wx.BoxSizer(wx.VERTICAL)
        
        choice_text = wx.StaticText(self, -1, 'Data loaders: ')
        self.choice_control = wx.Choice(self, -1, choices = plugin_list)
        self.choice_control.SetStringSelection(current_plugin)
        choice_sizer.Add(choice_text, 0, wx.ALIGN_CENTER_VERTICAL)
        choice_sizer.Add(self.choice_control, 0, wx.ALIGN_CENTER_VERTICAL)
        
        # Add the Dialog buttons
        button_sizer = wx.StdDialogButtonSizer()
        okay_button = wx.Button(self, wx.ID_OK)
        okay_button.SetDefault()
        button_sizer.AddButton(okay_button)
        apply_button = wx.Button(self, wx.ID_APPLY)
        apply_button.SetDefault()
        button_sizer.AddButton(apply_button)    
        button_sizer.AddButton(wx.Button(self, wx.ID_CANCEL))
        button_sizer.Realize()
        # Add some eventhandlers
        self.Bind(wx.EVT_BUTTON, self.on_apply, okay_button)
        self.Bind(wx.EVT_BUTTON, self.on_apply, apply_button)
        
        
        sizer.Add((20, 20), 0)
        sizer.Add(choice_sizer, 0, wx.ALIGN_CENTRE|wx.ALL)
        line = wx.StaticLine(self, -1, size=(20,-1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW|wx.TOP, 20)
        sizer.Add(button_sizer)
        
        self.SetSizer(sizer)
        
        sizer.Fit(self)
        self.Layout()
        
    def on_apply(self, event):
        if self.load_plugin_func is not None:
            self.load_plugin_func(self.choice_control.GetStringSelection())
        event.Skip()

# Utility Dialog functions..
def ShowInfoDialog(frame, message):
    dlg = wx.MessageDialog(frame, message,
                               'Information',
                               wx.OK | wx.ICON_INFORMATION
                               )
    dlg.ShowModal()
    dlg.Destroy()
    
def ShowErrorDialog(frame, message, position = ''):
    dlg = wx.MessageDialog(frame, message,
                               'ERROR',
                               wx.OK | wx.ICON_ERROR
                               )
    dlg.ShowModal()
    dlg.Destroy()

def ShowWarningDialog(frame, message):
    dlg = wx.MessageDialog(frame, message, 'Warning',
                               wx.OK | wx.ICON_ERROR
                               )
    dlg.ShowModal()
    dlg.Destroy()



''' <h1>Library for combined x-ray and neutrons simulations.</h1>
<p> The neutron simulations is capable of handling non-magnetic,
magnetic non-spin flip as well as neutron spin-flip reflectivity.
The model works with scattering lengths densities directly.
</p>
<h2>Classes</h2>
<h3>Layer</h3>
<code> Layer(sld_n=0.0, sld_x=1e-20J, d = 0.0, magn_ang = 0.0, sld_m = 0.0, sigma = 0.0)</code>
    <dl>
    <dt><code><b>sld_n</b></code></dt>
    <dd>The neutron scattering length density in 1e-6 1/AA^2</dd>
    <dt><code><b>d</b></code></dt>
    <dd>The thickness of the layer in AA (Angstroms = 1e-10m)</dd>
    <dt><code><b>sld_x</b></code></dt>
    <dd>The x-ray scattering length density in 1e-6 1/AA^2</dd>
    <dt><code><b>magn_ang</b></code></dt>
    <dd>The angle of the magnetic moment in degress. 0 degrees correspond to
    a moment collinear with the neutron spin.</dd>
    <dt><code><b>sld_m</b></code></dt>
    <dd>The neutron magnetic scattering length density in 1e-6 1/AA^2</dd>
    <dt><code><b>sigma</b></code></dt>
    <dd>The root mean square roughness of the top interface of the layer in Angstroms.</dd>
    </dl>
<h3>Stack</h3>
<code> Stack(Layers = [], Repetitions = 1)</code>
    <dl>
    <dt><code><b>Layers</b></code></dt>
    <dd>A <code>list</code> consiting of <code>Layer</code>s in the stack
    the first item is the layer closest to the bottom</dd>
    <dt><code><b>Repetitions</b></code></dt>
    <dd>The number of repetitions of the stack</dd>
    </dl>
<h3>Sample</h3>
<code> Sample(Stacks = [], Ambient = Layer(), Substrate = Layer())</code>
    <dl>
    <dt><code><b>Stacks</b></code></dt>
    <dd>A <code>list</code> consiting of <code>Stack</code>s in the stacks
    the first item is the layer closest to the bottom</dd>
    <dt><code><b>Ambient</b></code></dt>
    <dd>A <code>Layer</code> describing the Ambient (enviroment above the sample).
     Only the scattering lengths and density of the layer is used.</dd>
    <dt><code><b>Substrate</b></code></dt>
    <dd>A <code>Layer</code> describing the substrate (enviroment below the sample).
     Only the scattering lengths, density and  roughness of the layer is used.</dd>
    </dl>
    
<h3>Instrument</h3>
<code>Instrument(probe = 'x-ray', wavelength = 1.54, coords = 'tth',
     I0 = 1.0 res = 0.001, restype = 'no conv', respoints = 5, resintrange = 2,
     beamw = 0.01, footype = 'no corr', samplelen = 10.0, incangle = 0.0, pol = 'uu')</code>
    <dl>
    <dt><code><b>probe</b></code></dt>
    <dd>Describes the radiation and measurments used is one of:
    'x-ray', 'neutron', 'neutron pol', 'neutron pol spin flip'
    or the respective
    number 0, 1, 2, 3. The calculations for x-rays uses <code>sld_x</code> for the scattering
    length for neutrons <code>sld_n</code> for 'neutron pol', 'neutron pol spin flip' and
    alternatives the <code>sld_m</code>
    is used in the calculations. Note that the angle of magnetization <code>magn_ang</code>
    is only used in the last alternative.</dd>
    <dt><code><b>wavelength</b></code></dt>
    <dd>The wavalelngth of the radiation givenin AA (Angstroms)</dd>
    <dt><code><b>coords</b></code></dt>
    <dd>The coordinates of the data given to the SimSpecular function.
    The available alternatives are: 'q' or 'tth'. Alternatively the numbers
    0 (q) or 1 (tth) can be used.</dd>
    <dt><code><b>I0</b></code></dt>
    <dd>The incident intensity (a scaling factor)</dd>
    <dt><code><b>Ibkg</b></code></dt>
    <dd>The background intensity. Added as a constant value to the calculated
    reflectivity</dd>
    <dt><code><b>res</b></code></dt>
    <dd>The resolution of the instrument given in the coordinates of
     <code>coords</code>. This assumes a gaussian resolution function and
    <code>res</code> is the standard deviation of that gaussian.
    If <code>restype</code> has (dx/x) in its name the gaussian standard deviation is given by res*x where x is
    either in tth or q.</dd>
    <dt><code><b>restype</b></code></dt>
    <dd>Describes the rype of the resolution calculated. One of the alterantives:
    'no conv', 'fast conv', 'full conv and varying res.', 'fast conv + varying res.',
    'full conv and varying res. (dx/x)', 'fast conv + varying res. (dx/x)'.
    The respective numbers 0-3 also works. Note that fast convolution only alllows
    a single value into res wheras the other can also take an array with the
    same length as the x-data (varying resolution)</dd>
    <dt><code><b>respoints</b></code></dt>
    <dd>The number of points to include in the resolution calculation. This is only
    used for 'full conv and vaying res.', 'fast conv + varying res', 'full conv and varying res. (dx/x)' and
    'fast conv + varying res. (dx/x)'.</dd>
    <dt><code><b>resintrange</b></code></dt>
    <dd>Number of standard deviatons to integrate the resolution function times
    the reflectivity over</dd>
    <dt><code><b>footype</b></code></dt>
    <dd>Which type of footprint correction is to be applied to the simulation.
    One of: 'no corr', 'gauss beam' or 'square beam'. Alternatively, 
    the number 0-2 are also valid. The different choices are self expnalatory.</dd>
    <dt><code><b>beamw</b></code></dt>
    <dd>The width of the beam given in mm. For 'gauss beam' it should be
    the standard deviation. For 'square beam' it is the full width of the beam.</dd>
    <dt><code><b>samplelen</b></code></dt>
    <dd>The length of the sample given in mm</dd>
    <dt><code><b>incangle</b></code></dt>
    <dd>The incident angle of the neutrons, only valid in tof mode</dd>
    <dt><code><b>pol</b></code></dt>
    <dd>The measured polarization of the instrument. Valid options are:
    'uu','dd', 'ud', 'du' or 'ass' the respective number 0-3 also works.</dd>
'''
from numpy import *
from genx.gui_logging import iprint
from .lib import paratt as Paratt
from .lib import neutron_refl as MatrixNeutron
from .lib.instrument import *
from .lib import refl as refl
# Preamble to define the parameters needed for the models outlined below:

ModelID='SoftNX'
#InstrumentParameters={'Wavelength':1.54, 'Coordinates':1, 'I0':1.0, 'Sim': 0,\
#    'Res':0.001, 'Restype':0, 'Respoints':5, 'Resintrange':2, 'Beaw':0.01,\
#    'Footype':0.0, 'Samlen':10.0, 'Incangle':0.0}
__pars__ = ['Layer', 'Stack', 'Sample', 'Instrument']

instrument_string_choices = {'probe': ['x-ray', 'neutron', 'neutron pol',
    'neutron pol spin flip', 'neutron tof', 'neutron pol tof'], 'coords': ['q', 'tth'],
    'restype': ['no conv', 'fast conv',
     'full conv and varying res.', 'fast conv + varying res.',
     'full conv and varying res. (dx/x)', 'fast conv + varying res. (dx/x)'],
    'footype': ['no corr', 'gauss beam', 'square beam'],
    'pol': ['uu', 'dd', 'ud', 'ass', 'du']}
InstrumentParameters = {'probe':'x-ray', 'wavelength':1.54, 'coords':'tth',
                        'I0':1.0, 'res':0.001,
                        'restype':'no conv', 'respoints':5, 'resintrange':2, 'beamw':0.01,
                        'footype': 'no corr', 'samplelen':10.0, 'incangle':0.0, 'pol': 'uu',
                        'Ibkg': 0.0, 'tthoff':0.0}
InstrumentGroups = [('General', ['wavelength', 'coords', 'I0', 'Ibkg', 'tthoff']),
                    ('Resolution', ['restype', 'res', 'respoints', 'resintrange']),
                    ('Neutron', ['probe', 'pol', 'incangle']),
                    ('Footprint', ['footype', 'beamw', 'samplelen',]),
                    ]
InstrumentUnits = {'probe':'', 'wavelength': 'AA', 'coords':'',
                   'I0': 'arb.', 'res': '[coord]',
                   'restype':'', 'respoints':'pts.', 'resintrange':'[coord]', 'beamw':'mm',\
                   'footype': '', 'samplelen':'mm', 'incangle':'deg.', 'pol': '',\
                   'Ibkg': 'arb.', 'tthoff':'deg.'}
# Coordinates=1 or 'tth' => twothetainput
# Coordinates=0 or 'q'=> Q input
# probe: Type of simulation
#   'x-ray' or 0: X-rays (One output)
#   'neutron'or 1: Neutrons (One output, ignoring magn, magn_ang)
#   'neutron pol' or 2: Neutrons polarized (Two outputs Ruu,Rdd)
#   'neutron pol spin flip' or 3: Neutrons polarized with spin-flip 
#            (Three outputs Ruu,Rdd,Rud=Rdu, ignoring sigma!)
#   'neutron tof' or 4: Neutrons non-polarized TOF, Inc Angle must be set
#   'neutron pol tof'or 5: Neutrons polarized TOF (non-spin flip),
#         Inc Angle must be set
#
# res stddev of resolution
# restype 0 'none': No resolution convlution
#               1 or 'fast': Fast convolution
#               2 or 'full': Full Convolution +varying resolution
#               3 or 'ffull': Fast convolution varying resolution
# respoints Number of points for the convolution only valid for ResolutionType=2
# resintrange Number of standard deviatons to integrate over default 2
# Parameters for footprint coorections
# footype: 0 or 'no corr': No corections for footprint
#            1 or 'gauss beam': Correction for Gaussian beam => Beaw given in mm and stddev
#            2 or 'square beam': Correction for square profile => Beaw given in full width mm
# samlen= Samplelength in mm.

LayerParameters = {'sigma': 0.0, 'd': 0.0, 'sld_x': (1.0+1.0j)*1e-20,
                   'sld_n':  0.0 + 0.0J, 'sld_m': 0.0, 'magn_ang': 0.0}
LayerUnits = {'sigma': 'AA', 'd': 'AA', 'sld_x':'1e-6 1/AA^2',
              'sld_n': '1e-6 1/AA^2', 'sld_m': '1e-6 1/AA^2', 'magn_ang': 'deg.'}
LayerGroups = [('Standard', ['sld_x', 'd', 'sigma']),
               ('Neutron', ['sld_n', 'sld_m', 'magn_ang'])]
StackParameters = {'Layers': [], 'Repetitions': 1}
SampleParameters = {'Stacks': [], 'Ambient': None, 'Substrate': None}

AA_to_eV = 12398.5
''' Conversion from Angstrom to eV E = AA_to_eV/lamda.'''


q_limit = 1e-10
''' Minimum allowed q-value '''

# A buffer to save previous calculations for spin-flip calculations
class Buffer:
    Ruu = 0
    Rdd = 0
    Rdu = 0
    Rud = 0
    parameters = None
    TwoThetaQz = None


def footprintcorr(Q, instrument):
    foocor = 1.0
    footype = instrument.getFootype()
    beamw = instrument.getBeamw()
    samlen = instrument.getSamplelen()
    theta = arcsin(Q * instrument.getWavelength() / 4.0 / pi) * 180 / pi
    if footype == 1 or footype == instrument_string_choices['footype'][1]:
        foocor = GaussIntensity(theta, samlen / 2.0, samlen / 2.0, beamw)
    elif footype == 2 or footype == instrument_string_choices['footype'][2]:
        foocor = SquareIntensity(theta, samlen, beamw)
    elif footype == 0 or footype == instrument_string_choices['footype'][0]:
        pass
    else:
        raise ValueError('The choice of footprint correction, footype,'
                         'is WRONG')

    return foocor


def resolutioncorr(R, TwoThetaQz, foocor, instrument, weight):
    ''' Do the convolution of the reflectivity to account for resolution effects.'''
    restype = instrument.getRestype()
    if restype == instrument_string_choices['restype'][1] or restype == 1:
        R = ConvoluteFast(TwoThetaQz, R[:] * foocor, instrument.getRes(), \
                          range=instrument.getResintrange())
    elif (restype == instrument_string_choices['restype'][2] or restype == 2 or
          restype == instrument_string_choices['restype'][4] or restype == 4):
        R = ConvoluteResolutionVector(TwoThetaQz, R[:] * foocor, weight)
    elif restype == instrument_string_choices['restype'][3] or restype == 3:
        R = ConvoluteFastVar(TwoThetaQz, R[:] * foocor, instrument.getRes(), range=instrument.getResintrange())
    elif restype == instrument_string_choices['restype'][5] or restype == 5:
        R = ConvoluteFastVar(TwoThetaQz, R[:] * foocor, instrument.getRes()*TwoThetaQz,
                             range=instrument.getResintrange())
    elif restype == instrument_string_choices['restype'][0] or restype == 0:
        R = R[:] * foocor
    else:
        raise ValueError('The choice of resolution type, restype,'
                         'is WRONG')
    return R


def resolution_init(TwoThetaQz, instrument):
    ''' Inits the dependet variable with regards to coordinates and resolution.'''
    restype = instrument.getRestype()
    weight = 0
    if restype == 2 or restype == instrument_string_choices['restype'][2]:
        (TwoThetaQz, weight) = ResolutionVector(TwoThetaQz[:],
                                                instrument.getRes(), instrument.getRespoints(),
                                                range=instrument.getResintrange())
    elif restype == 4 or restype == instrument_string_choices['restype'][4]:
        (TwoThetaQz, weight) = ResolutionVector(TwoThetaQz[:],
                                                instrument.getRes()*TwoThetaQz, instrument.getRespoints(),
                                                range=instrument.getResintrange())
    # TTH values given as x
    if instrument.getCoords() == instrument_string_choices['coords'][1] \
            or instrument.getCoords() == 1:
        Q = 4 * pi / instrument.getWavelength() * sin((TwoThetaQz + instrument.getTthoff()) * pi / 360.0)
    # Q vector given....
    elif instrument.getCoords() == instrument_string_choices['coords'][0] \
            or instrument.getCoords() == 0:
        Q = 4 * pi / instrument.getWavelength() * sin(
            arcsin(TwoThetaQz * instrument.getWavelength() / 4 / pi) + instrument.getTthoff() * pi / 360.)
    else:
        raise ValueError('The value for coordinates, coords, is WRONG! should be q(0) or tth(1).')
    return Q, TwoThetaQz, weight


def Specular(TwoThetaQz, sample, instrument):
    """ Simulate the specular signal from sample when probed with instrument

    # BEGIN Parameters
    TwoThetaQz data.x
    # END Parameters
    """

    # preamble to get it working with my class interface
    restype = instrument.getRestype()
    Q, TwoThetaQz, weight = resolution_init(TwoThetaQz, instrument)
    if any(Q < q_limit):
        raise ValueError('The q vector has to be above %.1e'%q_limit)

    type = instrument.getProbe()
    pol = instrument.getPol()

    parameters = sample.resolveLayerParameters()
    if type ==  instrument_string_choices['probe'][0] or type==0:
        #fb = array(parameters['f'], dtype = complex64)
        e = AA_to_eV/instrument.getWavelength()
        sld = refl.cast_to_array(parameters['sld_x'], e)*1e-6
    else: 
        sld = array(parameters['sld_n'], dtype = complex128)*1e-6

    d = array(parameters['d'], dtype = float64)
    sld_m = array(parameters['sld_m'], dtype = float64)*1e-6
    #Transform to radians
    magn_ang = array(parameters['magn_ang'], dtype = float64)*pi/180.0 
    
    sigma = array(parameters['sigma'], dtype=float64)

    wl = instrument.getWavelength()
    l2pi = wl**2/2/3.141592
    # Ordinary Paratt X-rays
    if type == instrument_string_choices['probe'][0] or type == 0:
        R = Paratt.ReflQ(Q,instrument.getWavelength(), 1.0 - l2pi*sld, d, sigma)
    #Ordinary Paratt Neutrons
    elif type == instrument_string_choices['probe'][1] or type == 1:
        R = Paratt.ReflQ(Q,instrument.getWavelength(), 1.0 - l2pi*sld, d, sigma)
    #Ordinary Paratt but with magnetization
    elif type == instrument_string_choices['probe'][2] or type == 2:
        # Polarization uu or ++
        if pol == instrument_string_choices['pol'][0] or pol == 0:
            R = Paratt.ReflQ(Q,instrument.getWavelength(),\
                1.0 - l2pi*(sld + sld_m), d, sigma)
        # Polarization dd or --
        elif pol == instrument_string_choices['pol'][1] or pol == 1:
            R = Paratt.ReflQ(Q,instrument.getWavelength(),\
                 1.0 - l2pi*(sld - sld_m), d, sigma)
        elif pol == instrument_string_choices['pol'][3] or pol == 3:
            Rp = Paratt.ReflQ(Q, instrument.getWavelength(), 1.0 - l2pi*(sld - sld_m), d, sigma)
            Rm = Paratt.ReflQ(Q, instrument.getWavelength(), 1.0 - l2pi*(sld + sld_m), d, sigma)
            R = (Rp - Rm)/(Rp + Rm)

        else:
            raise ValueError('The value of the polarization is WRONG.'
                ' It should be uu(0) or dd(1)')
    # Spin flip
    elif type == instrument_string_choices['probe'][3] or type == 3:
        # Check if we have calcluated the same sample previous:
        if Buffer.TwoThetaQz is not None:
            Q_ok = Buffer.TwoThetaQz.shape == Q.shape
            if Q_ok:
                Q_ok = any(not_equal(Buffer.TwoThetaQz, Q))
        if Buffer.parameters != parameters or not Q_ok:
            #msld = 2.645e-5*magn*dens*instrument.getWavelength()**2/2/pi
            np = 1.0 - l2pi*(sld + sld_m)
            nm = 1.0 - l2pi*(sld - sld_m)
            Vp = (2*pi/instrument.getWavelength())**2*(1 - np**2)
            Vm = (2*pi/instrument.getWavelength())**2*(1 - nm**2)
            (Ruu, Rdd, Rud, Rdu) = MatrixNeutron.Refl(Q, Vp, Vm, d, magn_ang, sigma)
            Buffer.Ruu = Ruu; Buffer.Rdd = Rdd; Buffer.Rud = Rud
            Buffer.parameters = parameters.copy()
            Buffer.TwoThetaQz = Q.copy()
        else:
            pass
        # Polarization uu or ++
        if pol == instrument_string_choices['pol'][0] or pol == 0:
            R = Buffer.Ruu
        # Polarization dd or --
        elif pol == instrument_string_choices['pol'][1] or pol == 1:
            R = Buffer.Rdd
        # Polarization ud or +-
        elif (pol == instrument_string_choices['pol'][2] or pol == 2 or
              pol == instrument_string_choices['pol'][4] or pol == 4):
            R = Buffer.Rud
        # Calculating the asymmetry ass
        elif pol == instrument_string_choices['pol'][3] or pol == 3:
            R = (Buffer.Ruu - Buffer.Rdd)/(Buffer.Ruu + Buffer.Rdd + 2*Buffer.Rud)
        else:
            raise ValueError('The value of the polarization is WRONG. It should be uu(0), dd(1) or ud(2)')
    else:
        raise ValueError('The choice of probe is WRONG')
    #FootprintCorrections

    foocor = footprintcorr(Q, instrument)
    #Resolution corrections
    R = resolutioncorr(R, TwoThetaQz, foocor, instrument, weight)
    
    return R*instrument.getI0() + instrument.getIbkg()
    
def EnergySpecular(Energy, TwoThetaQz,sample,instrument):
    ''' Simulate the specular signal from sample when probed with instrument. Energy should be in eV.

    # BEGIN Parameters
    Energy data.x
    TwoThetaQz 3.0
    # END Parameters
    '''
    # preamble to get it working with my class interface
    restype = instrument.getRestype()
    #TODO: Fix so that resolution can be included.
    if restype != 0 and restype != instrument_string_choices['restype'][0]:
        raise ValueError('Only no resolution is allowed for energy scans.')

    wl = AA_to_eV/Energy

    # TTH values given as x
    if instrument.getCoords() == instrument_string_choices['coords'][1] \
            or instrument.getCoords() == 1:
        theta = TwoThetaQz/2.0
    # Q vector given....
    elif instrument.getCoords() == instrument_string_choices['coords'][0] \
            or instrument.getCoords() == 0:
        theta = arcsin(TwoThetaQz * wl / 4 / pi)*180.0/pi

    else:
        raise ValueError('The value for coordinates, coords, is WRONG!'
                         'should be q(0) or tth(1).')
    Q = 4 * pi / wl * sin((2*theta + instrument.getTthoff()) * pi / 360.0)

    type = instrument.getProbe()

    parameters = sample.resolveLayerParameters()
    if type ==  instrument_string_choices['probe'][0] or type==0:
        sld = refl.cast_to_array(parameters['sld_x'], Energy)*1e-6
    else:
        sld = array(parameters['sld_n'], dtype = complex64).real*1e-6

    d = array(parameters['d'], dtype = float64)
    sigma = array(parameters['sigma'], dtype = float64)

    wl = instrument.getWavelength()
    l2pi = wl**2/2/3.141592
    # Ordinary Paratt X-rays
    if type == instrument_string_choices['probe'][0] or type == 0:
        #R = Paratt.ReflQ(Q,instrument.getWavelength(),1.0-2.82e-5*sld,d,sigma)
        R = Paratt.Refl_nvary2(theta, wl, 1.0 - l2pi*sld, d, sigma)
    else:
        raise ValueError('The choice of probe is WRONG')
    #TODO: Fix corrections
    #FootprintCorrections
    #foocor = footprintcorr(Q, instrument)
    #Resolution corrections
    #R = resolutioncorr(R, TwoThetaQz, foocor, instrument, weight)

    return R*instrument.getI0() + instrument.getIbkg()


def OffSpecular(TwoThetaQz,ThetaQx,sample,instrument):
    ''' Function that simulates the off-specular signal (not implemented)
    
    # BEGIN Parameters
    TwoThetaQz 1.0
    ThetaQx data.x
    # END Parameters
    '''
    raise NotImplementedError('Not implemented use model interdiff insteads')
    return TwoThetaQz,ThetaQx

def SLD_calculations(z, item, sample, inst):
    ''' Calculates the scatteringlength density as at the positions z
    if item is None or "all" the function returns a dictonary of values.
    Otherwise it returns the item as identified by its string.
    
    # BEGIN Parameters
    z data.x
    item 'Re'
    # END Parameters
    '''
    parameters = sample.resolveLayerParameters()
    #f = array(parameters['f'], dtype = complex64)
    e = AA_to_eV/inst.getWavelength()
    sld_x = refl.cast_to_array(parameters['sld_x'], e)
    sld_n = array(parameters['sld_n'], dtype = complex64)
    type = inst.getProbe()
    magnetic = False
    mag_sld = 0
    sld_unit = '10^{-6}\AA^{2}'
    if type == instrument_string_choices['probe'][0] or type == 0:
        sld = sld_x
    elif type == instrument_string_choices['probe'][1] or type == 1 or\
        type == instrument_string_choices['probe'][4] or type == 4:
        sld = sld_n
        sld_unit = '10^{-6}/\AA^{2}'
    else:
        magnetic = True
        sld = sld_n
        sld_m = array(parameters['sld_m'], dtype = float64)
        #Transform to radians
        magn_ang = array(parameters['magn_ang'], dtype = float64)*pi/180.0
        mag_sld = sld_m
        mag_sld_x = mag_sld*cos(magn_ang)
        mag_sld_y = mag_sld*sin(magn_ang)
        sld_unit = '10^{-6}/\AA^{2}'
        
    d = array(parameters['d'], dtype = float64)
    d = d[1:-1]
    # Include one extra element - the zero pos (substrate/film interface)
    int_pos = cumsum(r_[0,d])
    sigma = array(parameters['sigma'], dtype = float64)[:-1] + 1e-7
    if z is None:
        z = arange(-sigma[0]*5, int_pos.max()+sigma[-1]*5, 0.5)
    if not magnetic:
        rho = sum((sld[:-1] - sld[1:])*(0.5 -\
            0.5*erf((z[:,newaxis]-int_pos)/sqrt(2.)/sigma)), 1) + sld[-1]
        dic = {'Re': real(rho), 'Im': imag(rho), 'z':z, 
               'SLD unit': sld_unit}
    else:
        sld_p = sld + mag_sld
        sld_m = sld - mag_sld
        rho_p = sum((sld_p[:-1] - sld_p[1:])*(0.5 -\
            0.5*erf((z[:,newaxis]-int_pos)/sqrt(2.)/sigma)), 1) + sld_p[-1]
        rho_m = sum((sld_m[:-1] - sld_m[1:])*(0.5 -\
            0.5*erf((z[:,newaxis]-int_pos)/sqrt(2.)/sigma)), 1)  + sld_m[-1]
        rho_mag_x = sum((mag_sld_x[:-1] - mag_sld_x[1:])*
                        (0.5 - 0.5*erf((z[:,newaxis]-int_pos)/sqrt(2.)/sigma)), 1) + mag_sld_x[-1]
        rho_mag_y = sum((mag_sld_y[:-1] - mag_sld_y[1:])*
                        (0.5 - 0.5*erf((z[:,newaxis]-int_pos)/sqrt(2.)/sigma)), 1) + mag_sld_y[-1]
        #dic = {'Re sld +': real(rho_p), 'Im sld +': imag(rho_p),\
        #        'Re sld -': real(rho_m), 'Im sld -': imag(rho_m), 'z':z,
        #        'SLD unit': sld_unit}
        rho_nucl = (rho_p + rho_m)/2.
        dic = {'Re non-mag': real(rho_nucl), 'Im non-mag': imag(rho_nucl),
                'mag': real(rho_p - rho_m)/2, 'mag_x': rho_mag_x, 'mag_y': rho_mag_y,
                'z':z, 'SLD unit': sld_unit}
    if item is None or item == 'all':
        return dic
    else:
        try:
            return dic[item]
        except:
            raise ValueError('The chosen item, %s, does not exist'%item)

SimulationFunctions={'Specular':Specular,
                     'OffSpecular':OffSpecular,
                     'SLD': SLD_calculations,
                     'EnergySpecular': EnergySpecular,
                    }


(Instrument, Layer, Stack, Sample) = refl.MakeClasses(InstrumentParameters,\
    LayerParameters, StackParameters, SampleParameters, SimulationFunctions,\
    ModelID)


if __name__=='__main__':
    pass



import numpy as np
from . import math_utils as mu
from . import isotropic_matrix as im
from . import int_lay_xmean
from genx.gui_logging import iprint
from functools import reduce

_ctype = np.complex128

def ass_P(k, d, ctype = _ctype):
    ''' Assamble the propagation matrix for the layers'''
    P = np.zeros((2,2) + k.shape, dtype = ctype)
    P[0,0] = np.exp(-1.0J*k*d)
    P[1,1] = 1.0/P[0,0]
    return P

def ass_X(k, ctype = _ctype):
    '''Assemble the interface matrix for all interfaces'''
    k_j = k[..., 1:]
    k_jm1 = k[..., :-1]

    X = np.empty((2, 2) + k_j.shape, dtype=ctype)
    
    X[0,0] = (k_j + k_jm1)/k_jm1/2
    X[0,1] = (k_jm1 - k_j)/k_jm1/2
    X[1,0] = X[0,1]
    X[1,1] = X[0,0]
    return X

def ass_X_2int(k_j, k_jm1, ctype = _ctype):
    '''Assemble the interface matrix for all interfaces'''
    #k_j = k[...,1:]
    #k_jm1 = k[...,:-1]

    X = np.empty((2,2) + k_j.shape, dtype = ctype)

    X[0,0] = (k_j + k_jm1)/k_jm1/2
    X[0,1] = (k_jm1 - k_j)/k_jm1/2
    X[1,0] = X[0,1]
    X[1,1] = X[0,0]
    return X

def ReflQ(Q, lamda, n, d, sigma):
    '''Calculate the reflectivity from a multilayer stack with
       Abeles matrix formalism. '''
    #pdb.set_trace()
    k_vac = 2*np.pi/lamda
    kz = Q[:, np.newaxis]/2.0
    n_amb2 = n[-1]*n[-1]
    k_j = np.sqrt((n*n - n_amb2)*k_vac*k_vac + n_amb2*kz*kz)
    
    #print d.shape, n.shape, k_j.dtype
    X = ass_X(k_j)
    #X = im.ass_X_interfacelayer4(k_j, k_j, k_j, k_j*0, k_j*0, sigma, sigma*0, sigma*0)
    P = ass_P(k_j, d)
    
    PX = mu.dot2_Adiag(P[...,1:-1], X[...,:-1])
    #print P.dtype, X.dtype, PX.dtype
    M = mu.dot2(X[...,-1], reduce(mu.dot2, rollaxis(PX, 3)[::-1]))
    
    r = M[1,0]/M[0,0]
    
    
    return np.abs(r)**2

def ReflQ_mag(Q, lamda, n, d, sigma, n_u, dd_u, sigma_u, n_l, dd_l, sigma_l):
    '''Calculate the reflectivity from a multilayer stack with
       Abeles matrix formalism.

       The last layer is the substrate and the first layer is the ambient
    '''
    #pdb.set_trace()
    k_vac = 2*np.pi/(lamda*np.ones(Q.shape))[:,np.newaxis]
    kz = Q[:, np.newaxis]/2.0
    n_amb2 = (n[:,0]*n[:,0])[:,np.newaxis]
    #print n_amb2.shape
    #print n.shape, n_l.shape, n_u.shape, kz.shape
    k = np.sqrt((n*n - n_amb2)*k_vac*k_vac + n_amb2*kz*kz)
    k_l = np.sqrt((n_l*n_l - n_amb2)*k_vac*k_vac + n_amb2*kz*kz)
    k_u = np.sqrt((n_u*n_u - n_amb2)*k_vac*k_vac + n_amb2*kz*kz)
    #print 'k: ', k.shape, k_l.shape, k_u.shape
    #print 'sigma_l', sigma_l.shape
    #print 'dd_u ', dd_u.shape
    #print np.all(n_u == n), np.all(n_l == n)
    #print np.all(k_ju == k_j), np.all(k_jl == k_j)
    
    #print d.shape, n.shape, k_j.dtype
    #X = ass_X(k)
    #X = im.ass_X_interfacelayer4(k, k_u, k_l, dd_u, dd_l, sigma, sigma_l, sigma_u)
    X_lu = ass_X_2int(k_u[:,1:], k_l[:,:-1])
    X_l = ass_X_2int(k_l[:,:-1], k[:,:-1])
    X_u = ass_X_2int(k[:,1:], k_u[:,1:])
    #print 'X: ', X_lu.shape, X_l.shape, X_u.shape

    X = int_lay_xmean.calc_iso_Xmean(X_l, X_lu, X_u, k, k_l, k_u, dd_u, dd_l,
                                     sigma, sigma_l, sigma_u)
    P = ass_P(k, d)
    
    PX = mu.dot2_Adiag(P[...,1:-1], X[...,1:])
    #print P.dtype, X.dtype, PX.dtype
    M = mu.dot2(X[...,0], reduce(mu.dot2, np.rollaxis(PX, 3)))
    
    r = M[1,0]/M[0,0]

    return np.abs(r)**2


def ReflQ_ref(Q,lamda,n,d,sigma):
    # Length of k-vector in vaccum
    d = d[1:-1]
    sigma = sigma[:-1]
    Q0 = 4*np.pi/lamda
    # Calculates the wavevector in each layer
    Qj=np.sqrt((n[:,np.newaxis]**2 - n[-1]**2)*Q0**2 + n[-1]**2*Q**2)
    # Fresnel reflectivity for the interfaces
    rp=(Qj[1:]-Qj[:-1])/(Qj[1:]+Qj[:-1])#*exp(-Qj[1:]*Qj[:-1]/2*sigma[:,newaxis]**2)
    #print rp.shape #For debugging
    #print d.shape
    #print Qj[1:-1].shape
    p=np.exp(1.0j*d[:,np.newaxis]*Qj[1:-1]) # Ignoring the top and bottom layer for the calc.
    #print p.shape #For debugging
    # Setting up a matrix for the reduce function. Reduce only takes one array
    # as argument
    rpp=np.array(list(map(lambda x,y:[x,y],rp[1:],p)))
    #print rpp.shape
    # Paratt's recursion formula
    def formula(rtot,rint):
        return (rint[0]+rtot*rint[1])/(1+rtot*rint[0]*rint[1])
    # Implement the recursion formula
    r=reduce(formula,rpp,rp[0])
    #print r.shape
    # return the reflectivity 
    return np.abs(r)**2


if __name__ == '__main__':
    from pylab import *
    atype = np.complex128
    atype = np.complex64
    theta = arange(0,5,0.01)+1e-13
    lamda = 1.54
    Q = 4*np.pi/lamda*np.sin(theta*np.pi/180)
    n_rep = 100
    n = array([1-7.57e-6+1.73e-7j,] + 
              [1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j]*n_rep +
              [1.0,], dtype=atype)
    d = array([2,] +
              [80,20]*n_rep +
              [2,], dtype=atype)
    n2 = array([1-7.57e-6+1.73e-7j,] + 
              [1-7.57e-6+1.73e-7j,1-2.24e-5+2.89e-6j]*n_rep +
              [1.0,], dtype=atype)
    d2 = array([2,] +
              [80,0]*n_rep +
              [2,], dtype=atype)
    n3 = array([1-7.57e-6+1.73e-7j,] + 
              [1-7.57e-6+1.73e-7j,1-7.57e-6+1.73e-7j]*n_rep +
              [1.0,], dtype=atype)
    d3 = array([2,] +
              [40,40]*n_rep +
              [2,], dtype=atype)
    
    sigma = array([0,] +
                  [0, 0]*n_rep +
                  [0])
    n_att = 1
    import time
    t1 = time.clock()
    for i in range(n_att):
        R_ref = ReflQ_ref(Q, 1.54, n, d, sigma)
        #R_ref = ReflQ(Q, 1.54, n2, d2, sigma)
    t2 = time.clock()
    for i in range(n_att):
        R_try = ReflQ(Q, 1.54, n, d, sigma)
        #R_try = ReflQ(Q, 1.54, n3, d3, sigma)
    t3 = time.clock()
    iprint('Paratt: ', (t2 -t1)/n_att)
    iprint('Abeles: ', (t3 - t2)/n_att)
    semilogy(theta, R_ref)
    semilogy(theta, R_try, '.')
    legend(('Parratt', 'Abeles with roughness/interface layer'))
    xlabel('Two Theta [deg.]')
    ylabel('Reflectivity')
    show()
